@@ -4,6 +4,9 @@
 /* define this to the version number of the program */
 #undef VERSION
 
+/* define if you are using rpm >= 4.0 */
+#undef HAVE_RPM_4_0
+
 /* define if you are using rpm >= 2.5 */
 #undef HAVE_RPM_3_0
 
