@@ -3,7 +3,7 @@
  *
  * See Copyright for the status of this software.
  *
- * $Id: rdf.c,v 1.1.1.1 1999/03/13 02:26:43 jamesh Exp $
+ * $Id: rdf.c,v 1.7 2000/09/14 13:03:38 malcolm Exp $
  */
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -31,13 +31,14 @@
 #include "trans.h"
 #include "distrib.h"
 
+extern ComplainFunc complain;
+
 /*
  * Open an RDF file call the parser to create a XML tree
  * Then walk the tree and build an rpmData structure for
  * the corresponding package.
  */
 rpmData *rpmOpenRdfFile(char *file) {
-    char nameBuffer[200];
     rdfSchema rdf;
     rdfNamespace rpmNs;
     rdfNamespace rdfNs;
@@ -48,7 +49,6 @@ rpmData *rpmOpenRdfFile(char *file) {
     char *name;
     rpmData *rpm;
     struct stat buf;
-    int i;
 
     rdf = rdfRead(file);
     if (rdf == NULL) return(NULL);
@@ -58,19 +58,19 @@ rpmData *rpmOpenRdfFile(char *file) {
      */
     rdfNs = rdfGetNamespace(rdf, "http://www.w3.org/TR/WD-rdf-syntax#");
     if (rdfNs == NULL) {
-        g_warning("%s is not an RDF schema", file);
+        complain(_("%s is not an RDF schema"), file);
 	rdfDestroySchema(rdf);
 	return(NULL);
     }
     rpmNs = rdfGetNamespace(rdf, "http://www.rpm.org/");
     if (rdfNs == NULL) {
-        g_warning("%s is not an RPM specific RDF schema", file);
+        complain(_("%s is not an RPM specific RDF schema"), file);
 	rdfDestroySchema(rdf);
 	return(NULL);
     }
     desc = rdfFirstDescription(rdf);
     if (rdfNs == NULL) {
-        g_warning("%s RDF schema seems empty", file);
+        complain(_("%s RDF schema seems empty"), file);
 	rdfDestroySchema(rdf);
 	return(NULL);
     }
@@ -81,7 +81,7 @@ rpmData *rpmOpenRdfFile(char *file) {
      */
     rpm = g_new0(rpmData, 1);
     if (rpm == NULL) {
-        g_warning("rpmOpenRdf : out of memory !");
+        complain(_("rpmOpenRdf : out of memory !"));
 	rdfDestroySchema(rdf);
 	return(NULL);
     }
@@ -96,7 +96,7 @@ rpmData *rpmOpenRdfFile(char *file) {
     rdfGetValue(desc, "Name", rpmNs, &value, NULL);
     if (value != NULL) rpm->name = value;
     else {
-        g_warning("%s RDF schema invalid : no Name", file);
+        complain(_("%s RDF schema invalid : no Name"), file);
 	rdfDestroySchema(rdf);
 	return(NULL);
     }
@@ -105,7 +105,7 @@ rpmData *rpmOpenRdfFile(char *file) {
       rpm->version = g_strdup(value);
       free(value);
     } else {
-        g_warning("%s RDF schema invalid : no Version", file);
+        complain(_("%s RDF schema invalid : no Version"), file);
 	rdfDestroySchema(rdf);
 	return(NULL);
     }
@@ -114,7 +114,7 @@ rpmData *rpmOpenRdfFile(char *file) {
       rpm->release = g_strdup(value);
       free(value);
     } else {
-        g_warning("%s RDF schema invalid : no Release", file);
+        complain(_("%s RDF schema invalid : no Release"), file);
 	rdfDestroySchema(rdf);
 	return(NULL);
     }
@@ -239,14 +239,14 @@ rpmData *rpmOpenRdfFile(char *file) {
 						   g_strdup(value));
 		    free(value);
 		} else
-		  g_warning("%s: malformed Resource element", file);
+		  complain(_("%s: malformed Resource element"), file);
 	    } else
-	      g_warning("%s: malformed Provides bag", file);
+	      complain(_("%s: malformed Provides bag"), file);
 	    if (name != NULL) free(name);
 	    elem = rdfNextElem(elem);
 	}
     } else
-        g_warning("%s: doesn't export any resource", file);
+        complain(_("%s: doesn't export any resource"), file);
 
     /*
      * idem for the dependencies.
@@ -268,9 +268,9 @@ rpmData *rpmOpenRdfFile(char *file) {
 						  g_strdup(value));
 		    free(value);
 		} else
-		  g_warning("%s: malformed Resource element", file);
+		  complain(_("%s: malformed Resource element"), file);
 	    } else
-	      g_warning("%s: malformed Provides bag", file);
+	      complain(_("%s: malformed Provides bag"), file);
 	    if (name != NULL) free(name);
 	    elem = rdfNextElem(elem);
 	}
@@ -297,6 +297,8 @@ rpmData *rpmOpenPackage(char *distID, char *name, char *version,
   rpmData *rpm;
 
   g_free(tmp);
+  if(file==NULL)
+	return NULL;
   rpm = rpmOpenRdfFile(file);
   rpm->subdir = g_strdup(distID);
   unlink(file);
@@ -309,7 +311,7 @@ char *rpmDataGetURL(rpmData *rpm) {
   char *part = rpm->subdir + strlen(distrib->ID);
 
   if (!distrib) {
-    g_warning("unknown distribution %s", rpm->subdir?rpm->subdir:"(null)");
+    complain(_("Unknown distribution %s"), rpm->subdir?rpm->subdir:"(null)");
     return NULL;
   }
   return g_strconcat(distrib->myMirror, part, "/",  rpm->name, "-",
