@@ -41,6 +41,7 @@
 /* undef this if you don't want to cache rpmData structures */
 #define RPM_DATA_CACHE
 
+extern ComplainFunc complain;
 extern GVoidFunc libfind_idle_func;
 
 /* --- These functions form a SAX parser for the fullIndex.rdf index -- */
@@ -147,13 +148,13 @@ static void searchStartElement(SearchParseState *state, const CHAR *name,
     g_assert(state->name != NULL);
     g_assert(state->summary != NULL);
     if (strcmp(name, state->rdf))
-      g_warning("expected %s as toplevel element -- got %s",state->rdf,name);
+      complain(_("Expected %s as toplevel element -- got %s"),state->rdf,name);
 
     state->state = PARSER_IN_RDF;
     break;
   case PARSER_IN_RDF:
     if (strcmp(name, state->description))
-      g_warning("was not expecting a %s element", name);
+      complain(_("Was not expecting a %s element"), name);
     g_string_truncate(state->nameData, 0);
     g_string_truncate(state->summaryData, 0);
     state->state = PARSER_IN_DESCRIPTION;
@@ -166,18 +167,18 @@ static void searchStartElement(SearchParseState *state, const CHAR *name,
       g_string_truncate(state->summaryData, 0);
       state->state = PARSER_IN_SUMMARY;
     } else
-      g_warning("was not expecting a %s element", name);
+      complain(_("Was not expecting a %s element"), name);
     break;
   case PARSER_IN_NAME:
   case PARSER_IN_SUMMARY:
-    g_warning("was not expecting a %s element", name);
+    complain(_("was not expecting a %s element"), name);
     break;
   }
 }
 static void searchEndElement(SearchParseState *state, const CHAR *name) {
   switch (state->state) {
   case PARSER_START:
-    g_warning("This state should not have been reached");
+    complain(_("This state should not have been reached"));
     break;
   case PARSER_IN_RDF:
     break;
@@ -380,7 +381,7 @@ GList *alternateSearch(const char *resource) {
 
   g_free(tmp);
   if (file == NULL) {
-    g_warning("couldn't get info for resource %s", resource);
+    complain(_("Couldn't get info for resource %s"), resource);
     return NULL;
   }
   rdf = rdfRead(file);
@@ -391,19 +392,19 @@ GList *alternateSearch(const char *resource) {
 
   rdfNs = rdfGetNamespace(rdf, "http://www.w3.org/TR/WD-rdf-syntax#");
   if (rdfNs == NULL) {
-    g_warning("%s not an RDF schema", resource);
+    complain(_("%s not an RDF schema"), resource);
     rdfDestroySchema(rdf);
     return NULL;
   }
   rpmNs = rdfGetNamespace(rdf, "http://www.rpm.org/");
   if (rpmNs == NULL) {
-    g_warning("%s not an RPM specific RDF schema", resource);
+    complain(_("%s not an RPM specific RDF schema"), resource);
     rdfDestroySchema(rdf);
     return NULL;
   }
   desc = rdfFirstDescription(rdf);
   if (desc == NULL) {
-    g_warning("%s RDF schema seems empty", resource);
+    complain(_("%s RDF schema seems empty"), resource);
     rdfDestroySchema(rdf);
     return NULL;
   }
@@ -413,7 +414,7 @@ GList *alternateSearch(const char *resource) {
 
     if (value != NULL) URL = value;
     else {
-      g_warning("%s RDF schema invalid: description without href", resource);
+      complain(_("%s RDF schema invalid: description without href"), resource);
       rdfDestroySchema(rdf);
       freeAlternates(ret);
       return NULL;
@@ -426,7 +427,7 @@ GList *alternateSearch(const char *resource) {
       alt->name = g_strdup(value);
       free(value);
     } else {
-      g_warning("%s RDF schema invalid: no name", resource);
+      complain(_("%s RDF schema invalid: no name"), resource);
       rdfDestroySchema(rdf);
       freeRpmPackageAlternate(alt);
       freeAlternates(ret);
@@ -437,7 +438,7 @@ GList *alternateSearch(const char *resource) {
       alt->version = g_strdup(value);
       free(value);
     } else {
-      g_warning("%s RDF schema invalid: no version", resource);
+      complain(_("%s RDF schema invalid: no version"), resource);
       rdfDestroySchema(rdf);
       freeRpmPackageAlternate(alt);
       freeAlternates(ret);
@@ -448,7 +449,7 @@ GList *alternateSearch(const char *resource) {
       alt->release = g_strdup(value);
       free(value);
     } else {
-      g_warning("%s RDF schema invalid: no release", resource);
+      complain(_("%s RDF schema invalid: no release"), resource);
       rdfDestroySchema(rdf);
       freeRpmPackageAlternate(alt);
       freeAlternates(ret);
@@ -605,6 +606,25 @@ void statAlternates(rpmdb db, GList *alternates) {
 
     /* a slight optimisation, since usually all alternates have the same name*/
     if (!prevname || strcmp(prevname, alt->name) != 0) {
+
+#ifdef	HAVE_RPM_4_0
+      rpmdbMatchIterator mi;
+      Header tmp;
+
+      if (installed) headerFree(installed);
+      installed = NULL;
+      mi = rpmdbInitIterator(db, RPMDBI_LABEL, alt->name, 0);
+      while ((tmp = rpmdbNextIterator(mi)) != NULL) {
+	if (!installed) installed = headerLink(tmp);
+	else {
+	  if (rpmVersionCompare(installed, tmp) < 0) {
+	    headerFree(installed);
+	    installed = headerLink(tmp);
+	  }
+	}
+      }
+      rpmdbFreeIterator(mi);
+#else	/* HAVE_RPM_4_0 */
       dbiIndexSet matches;
     
       if (installed) headerFree(installed);
@@ -627,7 +647,10 @@ void statAlternates(rpmdb db, GList *alternates) {
 	}
 	dbiFreeIndexRecord(matches);
       }
+#endif	/* HAVE_RPM_4_0 */
+
     }
+
     /* installed holds the header for the newest version of the package,
      * or is NULL if the package is not installed */
     if (!installed)
@@ -683,8 +706,11 @@ rpmData *alternateGetInfo(rpmPackageAlternate *alt) {
   if (!ret) {
     ret = rpmOpenPackage(alt->subdir, alt->name, alt->version,
 			 alt->release, alt->arch);
-    ret->nofree = TRUE;
-    g_hash_table_insert(cache, g_strdup(alt->key), ret);
+    if(ret!=NULL)
+    {
+      ret->nofree = TRUE;
+      g_hash_table_insert(cache, g_strdup(alt->key), ret);
+    }
   }
   return ret;
 #else
