@@ -75,6 +75,7 @@ static enum { MODE_UNSET, MODE_QUERY, MODE_INSTALL, MODE_UPGRADE,
   majorMode = MODE_UNSET;
 static int isFiles = FALSE;
 static GdkWindow *main_window = NULL;
+static int needroot = 0;
 
 static void parseAnArg(poptContext con, enum poptCallbackReason reason,
 		       const struct poptOption *opt,
@@ -93,11 +94,11 @@ static void parseAnArg(poptContext con, enum poptCallbackReason reason,
     case -4: /* -p */
       isFiles = TRUE; break;
     case -5: /* --install */
-      majorMode = MODE_INSTALL; break;
+      majorMode = MODE_INSTALL; needroot=1; break;
     case -6: /* --upgrade */
-      majorMode = MODE_UPGRADE; break;
+      majorMode = MODE_UPGRADE; needroot=1; break;
     case -7: /* --verify */
-      majorMode = MODE_VERIFY; break;
+      majorMode = MODE_VERIFY; needroot=1; break;
     case -8: /* --checksig */
       majorMode = MODE_CHECKSIG;
     }
@@ -128,30 +129,30 @@ static void query_verify_func(RpmQuery *info, gpointer data) {
 static void query_uninstall_func(RpmQuery *info, gpointer data) {
   int failed;
   char buf[512];
-#ifdef HAVE_RPM_3_0
-  guint interfaceFlags = 0, transFlags = 0, probFilter = 0;
-  rpm_props_box_get_flags(&interfaceFlags, &transFlags, &probFilter);
-#else    
+#ifdef HAVE_RPM_2_5
   guint interfaceFlags = 0, installFlags = 0, uninstallFlags = 0;
   rpm_props_box_get_flags(&interfaceFlags, &installFlags, &uninstallFlags);
+#else    
+  guint interfaceFlags = 0, transFlags = 0, probFilter = 0;
+  rpm_props_box_get_flags(&interfaceFlags, &transFlags, &probFilter);
 #endif
 
-#ifdef HAVE_RPM_3_0
-  failed = uninstall_one(info->hdl->root, info->index, transFlags,
-                         probFilter, interfaceFlags);
-#else
+#ifdef HAVE_RPM_2_5
   failed = uninstall_one(info->hdl->root, info->index, uninstallFlags,
                          interfaceFlags);
+#else
+  failed = uninstall_one(info->hdl->root, info->index, transFlags,
+                         probFilter, interfaceFlags);
 #endif
   if (failed) {
     g_snprintf(buf, 511, _("%d packages couldn't be uninstalled"), failed);
     message_box(buf);
     return;
   }
-#ifdef HAVE_RPM_3_0
-  if (!(transFlags & RPMTRANS_FLAG_TEST)) {
+#ifdef HAVE_RPM_2_5
+  if (!(uninstallFlags & RPMUNINSTALL_TEST)) {
 #else
-    if (!(uninstallFlags & RPMUNINSTALL_TEST)) {
+  if (!(transFlags & RPMTRANS_FLAG_TEST)) {
 #endif
     GtkWidget *parent = GTK_WIDGET(info)->parent;
 
@@ -159,19 +160,19 @@ static void query_uninstall_func(RpmQuery *info, gpointer data) {
   }
 }
 static void query_install_func(RpmQuery *info, gpointer data) {
-#ifdef HAVE_RPM_3_0
-  guint interfaceFlags = 0, transFlags = 0, probFilter = 0;
-  rpm_props_box_get_flags(&interfaceFlags, &transFlags, &probFilter);
-#else    
+#ifdef HAVE_RPM_2_5
   guint interfaceFlags = 0, installFlags = 0, uninstallFlags = 0;
   rpm_props_box_get_flags(&interfaceFlags, &installFlags, &uninstallFlags);
+#else    
+  guint interfaceFlags = 0, transFlags = 0, probFilter = 0;
+  rpm_props_box_get_flags(&interfaceFlags, &transFlags, &probFilter);
 #endif
 
-#ifdef HAVE_RPM_3_0
-  if (install_one(info->hdl->root, info->fname, NULL, transFlags, probFilter,
+#ifdef HAVE_RPM_2_5
+  if (install_one(info->hdl->root, info->fname, NULL, installFlags,
                   interfaceFlags, NULL, NULL)) {
 #else
-  if (install_one(info->hdl->root, info->fname, NULL, installFlags,
+  if (install_one(info->hdl->root, info->fname, NULL, transFlags, probFilter,
                   interfaceFlags, NULL, NULL)) {
 #endif
     gchar buf[512];
@@ -180,21 +181,21 @@ static void query_install_func(RpmQuery *info, gpointer data) {
   }
 }
 static void query_upgrade_func(RpmQuery *info, gpointer data) {
-#ifdef HAVE_RPM_3_0
-  guint interfaceFlags = 0, transFlags = 0, probFilter = 0;
-  rpm_props_box_get_flags(&interfaceFlags, &transFlags, &probFilter);
-#else    
+#ifdef HAVE_RPM_2_5
   guint interfaceFlags = 0, installFlags = 0, uninstallFlags = 0;
   rpm_props_box_get_flags(&interfaceFlags, &installFlags, &uninstallFlags);
+#else    
+  guint interfaceFlags = 0, transFlags = 0, probFilter = 0;
+  rpm_props_box_get_flags(&interfaceFlags, &transFlags, &probFilter);
 #endif
 
-#ifdef HAVE_RPM_3_0
+#ifdef HAVE_RPM_2_5
   if (install_one(info->hdl->root, info->fname, NULL,
-                  transFlags, probFilter, interfaceFlags | INTER_UPGRADE,
+                  installFlags | RPMINSTALL_UPGRADE, interfaceFlags,
                   NULL, NULL)) {
 #else
   if (install_one(info->hdl->root, info->fname, NULL,
-                  installFlags | RPMINSTALL_UPGRADE, interfaceFlags,
+                  transFlags, probFilter, interfaceFlags | INTER_UPGRADE,
                   NULL, NULL)) {
 #endif      
     gchar buf[512];
@@ -212,13 +213,12 @@ int main(int argc, char *argv[]) {
   GnomeClient *client;
   DBHandle *dbhdl = NULL;
   GList *pkgs = NULL, *tmp;
-  dbiIndexSet matches;
-  gint i, res;
+  gint res;
   gchar buf[512];
-#ifdef HAVE_RPM_3_0
-  int interfaceFlags = 0;
-#else
+#ifdef HAVE_RPM_2_5
   int installFlags = 0;
+#else
+  int interfaceFlags = 0;
 #endif
 
   /* i18n stuff */
@@ -233,11 +233,18 @@ int main(int argc, char *argv[]) {
   gtk_signal_connect(GTK_OBJECT(client), "die",
 		     GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
 
-#ifdef HAVE_RPM_3_0 /* RPM_3_0 */
-  rpmReadConfigFiles(NULL, NULL);
-#else
+  if(geteuid() && needroot)
+  {
+    GtkWidget *d=gnome_error_dialog(_("You need to be the superuser to use this application."));
+    gnome_dialog_run_and_close(GNOME_DIALOG(d));
+    gtk_exit(0);
+  }
+
+#ifdef HAVE_RPM_2_5
   if (rpmReadConfigFiles(NULL, NULL, NULL, 0))
     return 1;
+#else
+  rpmReadConfigFiles(NULL, NULL);
 #endif
   setupErrorHandler();
 
@@ -276,6 +283,18 @@ int main(int argc, char *argv[]) {
     } else {
       pkgs = NULL;
       for (tmp = args; tmp != NULL; tmp = tmp->next) {
+#ifdef	HAVE_RPM_4_0
+	rpmdbMatchIterator mi;
+
+	mi = rpmdbInitIterator(dbhdl->db, RPMDBI_LABEL, tmp->data, 0);
+	while (rpmdbNextIterator(mi) != NULL)
+	  pkgs = g_list_append(pkgs,
+				 GINT_TO_POINTER(rpmdbGetIteratorOffset(mi)));
+	rpmdbFreeIterator(mi);
+#else	/* HAVE_RPM_4_0 */
+ 	dbiIndexSet matches;
+	gint i;
+
 	if (rpmdbFindByLabel(dbhdl->db, tmp->data, &matches))
 	  continue;
 	for (i = 0; i < matches.count; i++)
@@ -283,6 +302,7 @@ int main(int argc, char *argv[]) {
 	    pkgs = g_list_append(pkgs,
 				 GINT_TO_POINTER(matches.recs[i].recOffset));
 	dbiFreeIndexRecord(matches);
+#endif	/* HAVE_RPM_4_0 */
       }
       if (!pkgs) {
 	message_box(_("You must give a package name to query"));
@@ -314,16 +334,16 @@ int main(int argc, char *argv[]) {
     gtk_main();
     break;
   case MODE_UPGRADE:
-#ifdef HAVE_RPM_3_0
-    interfaceFlags = interfaceFlags | INTER_UPGRADE;
-#else
+#ifdef HAVE_RPM_2_5
     installFlags = installFlags | RPMINSTALL_UPGRADE;
+#else
+    interfaceFlags = interfaceFlags | INTER_UPGRADE;
 #endif
   case MODE_INSTALL:
-#ifdef HAVE_RPM_3_0
-    res = do_install(root, args, NULL, 0, 0, interfaceFlags, NULL, NULL);
-#else
+#ifdef HAVE_RPM_2_5
     res = do_install(root, args, NULL, installFlags, 0, NULL, NULL);
+#else
+    res = do_install(root, args, NULL, 0, 0, interfaceFlags, NULL, NULL);
 #endif
     if (res) {
       g_snprintf(buf, 511, _("%s of %d packages failed."),
@@ -335,6 +355,18 @@ int main(int argc, char *argv[]) {
     db_handle_db_up(dbhdl);
     pkgs = NULL;
     for (tmp = args; tmp != NULL; tmp = tmp->next) {
+#ifdef	HAVE_RPM_4_0
+	rpmdbMatchIterator mi;
+
+	mi = rpmdbInitIterator(dbhdl->db, RPMDBI_LABEL, tmp->data, 0);
+	while (rpmdbNextIterator(mi) != NULL)
+	  pkgs = g_list_append(pkgs,
+				 GINT_TO_POINTER(rpmdbGetIteratorOffset(mi)));
+	rpmdbFreeIterator(mi);
+#else	/* HAVE_RPM_4_0 */
+      dbiIndexSet matches;
+      gint i;
+
       if (rpmdbFindByLabel(dbhdl->db, tmp->data, &matches))
 	continue;
       for (i = 0; i < matches.count; i++)
@@ -342,6 +374,7 @@ int main(int argc, char *argv[]) {
 	  pkgs = g_list_append(pkgs,
 			       GINT_TO_POINTER(matches.recs[i].recOffset));
       dbiFreeIndexRecord(matches);
+#endif	/* HAVE_RPM_4_0 */
     }
     if (!pkgs) {
       message_box(_("You must give a package name to query"));
@@ -374,13 +407,12 @@ static gint save_state(GnomeClient *client, gint phase,
 		       GnomeRestartStyle save_style, gint shutdown,
 		       GnomeInteractStyle interact_style, gint fast,
 		       gchar *prog_name) {
-  gint x, y, w, h;
+  gint i = 0;
   gchar *geom = NULL;
   if (main_window)
     geom = gnome_geometry_string(main_window);
   if (majorMode == MODE_UNSET) {
     gchar *argv[6];
-    gint i = 0;
 
     argv[i++] = prog_name;
     if (root && *root) {
@@ -396,7 +428,6 @@ static gint save_state(GnomeClient *client, gint phase,
   } else if (majorMode == MODE_QUERY) {
     gchar **argv = g_new(gchar *, 7 + g_list_length(args));
     GList *tmp;
-    gint i = 0;
     argv[i++] = prog_name;
     if (root && *root) {
       argv[i++] = "--root";
