@@ -41,21 +41,26 @@
 #define UNKNOWN_SIZE 1024
 
 static RpmProgress *prog;
+
+#ifdef HAVE_RPM_2_5
+
 static rpmInstallCb inst_cb;
 static void *inst_cb_user_data;
-#ifdef HAVE_RPM_3_0
+
+static void updateDisp(gulong amount, gulong total) {
+  rpm_progress_update(prog, (1.0 * amount / total));
+}
+
+#else	/* !HAVE_RPM_2_5 */
+
 static void * updateDisp(const Header h, const rpmCallbackType what,
 			 const unsigned long amount, const unsigned long total,
 			 const void * pkgKey, void * data)
 {
-  char * s;
-  int flags = (int) data;
-  void * rc = NULL;
   static FD_t fd;
   gint size;
   unsigned long *sizep;
   char *name;
-  Header head;
   const char * filename = pkgKey;
 
   switch (what) {
@@ -83,13 +88,9 @@ static void * updateDisp(const Header h, const rpmCallbackType what,
   }
   return NULL;
 }
-#else  /* !HAVE_RPM_3_0 */
-static void updateDisp(gulong amount, gulong total) {
-  rpm_progress_update(prog, (1.0 * amount / total));
-}
 #endif
 
-#ifndef HAVE_RPM_3_0
+#ifdef HAVE_RPM_2_5
 
 static gint installPackages(gchar *rootdir, gchar **packages, gchar *location,
                        gint numPackages, gint totSize, gint installFlags,
@@ -182,7 +183,6 @@ static void addDepNodes(rpmdb db, gchar *rootdir,
   gint i, flags;
   guint32 offs;
   gchar buf[1024], *row[1];
-  dbiIndexSet matches;
   struct rpmDependencyConflict *conf2;
   gint numConf2;
 
@@ -216,28 +216,36 @@ static void addDepNodes(rpmdb db, gchar *rootdir,
 				 NULL, NULL, NULL, NULL, FALSE, FALSE);
 
     /* get the record offset for this package ... */
+    offs = 0;
+#ifdef	HAVE_RPM_4_0
+  { rpmdbMatchIterator mi;
+    const char * name, * version, * release;
+    headerNVR(conflicts[i].byHeader, &name, &version, &release);
+    mi = rpmdbInitIterator(db, RPMTAG_NAME, name, 0);
+    rpmdbSetIteratorVersion(mi, version);
+    rpmdbSetIteratorRelease(mi, release);
+    while (rpmdbNextIterator(mi)) {
+      offs = rpmdbGetIteratorOffset(mi);
+      break;
+    }
+    rpmdbFreeIterator(mi);
+  }
+#else
+  { dbiIndexSet matches;
     if (rpmdbFindByHeader(db, conflicts[i].byHeader, &matches))
       /* could not find header ... */
       continue;
     offs = matches.recs[0].recOffset;
     dbiFreeIndexRecord(matches);
+  }
+#endif
     if (!offs || g_list_find(*processed, GUINT_TO_POINTER(offs)))
 	/* already processed this package */
 	continue;
     *processed = g_list_prepend(*processed, GUINT_TO_POINTER(offs));
 
     /* get a conflicts array for this package */
-#ifdef HAVE_RPM_3_0
-    {
-      rpmTransactionSet rpmdep = rpmtransCreateSet(db, rootdir);
-      rpmtransRemovePackage(rpmdep, offs);
-      if (rpmdepCheck(rpmdep, &conf2, &numConf2)) {
-	rpmtransFree(rpmdep);
-	continue;
-      }
-      rpmtransFree(rpmdep);
-    }
-#else
+#ifdef HAVE_RPM_2_5
     {
       rpmDependencies rpmdep = rpmdepDependencies(db);
       rpmdepRemovePackage(rpmdep, offs);
@@ -247,6 +255,16 @@ static void addDepNodes(rpmdb db, gchar *rootdir,
       }
       rpmdepDone(rpmdep);
     }
+#else
+    {
+      rpmTransactionSet rpmdep = rpmtransCreateSet(db, rootdir);
+      rpmtransRemovePackage(rpmdep, offs);
+      if (rpmdepCheck(rpmdep, &conf2, &numConf2)) {
+	rpmtransFree(rpmdep);
+	continue;
+      }
+      rpmtransFree(rpmdep);
+    }
 #endif
     /* add nodes for packages this one depends on ... */
     addDepNodes(db, rootdir, ctree, node, conf2, numConf2, processed);
@@ -261,8 +279,7 @@ static gboolean displayDepProblems(struct rpmDependencyConflict *conflicts,
 				   rpmdb db, gchar *rootdir,
 				   GList **dependentPkgs) {
   GtkWidget *win, *w, *ctree;
-  gchar buf[1024];
-  gint i, flags, ret = FALSE;
+  gint ret = FALSE;
   GList *processed = NULL;
 
   win = gnome_dialog_new(_("Dependency Problems"), GNOME_STOCK_BUTTON_YES,
@@ -310,13 +327,116 @@ static gboolean displayDepProblems(struct rpmDependencyConflict *conflicts,
   return !ret;
 }
 
-#ifdef HAVE_RPM_3_0
+#ifdef HAVE_RPM_2_5
+
+int do_install(gchar *rootdir, GList *pkgs, gchar *location,
+	       gint installFlags, gint interfaceFlags,
+	       rpmInstallCb cb, void *user_data) {
+  gint mode, rc, i, fd, isSource, stopInstall = 0, pkgc;
+  gint numPackages, numBinaryPackages = 0;
+  gint numSourcePackages = 0, numFailed = 0;
+  gint numConflicts, totSize = 0;
+  gchar **packages, *pkg_file;
+  Header *binaryHeaders;
+  rpmdb db;
+  rpmDependencies rpmdep;
+  struct rpmDependencyConflict *conflicts;
+  gulong *sizep;
+
+  if (installFlags & RPMINSTALL_TEST)
+    mode = O_RDONLY;
+  else
+    mode = O_RDWR | O_CREAT;
+
+  inst_cb = cb;
+  inst_cb_user_data = user_data;
+
+  pkgc = g_list_length(pkgs);
+  packages = g_new(gchar *, pkgc + 1);
+  binaryHeaders = g_new(Header, pkgc + 1);
+  for (numPackages = 0; pkgs != NULL; pkgs = pkgs->next) {
+    pkg_file = pkgs->data;
+    fd = open(pkg_file, O_RDONLY);
+    if (fd < 0) {
+      numFailed++;
+      continue;
+    }
+    packages[numPackages++] = pkg_file;
+    rc = rpmReadPackageHeader(fd, &binaryHeaders[numBinaryPackages],
+                              &isSource, NULL, NULL);
+    close(fd);
+    if (binaryHeaders[numBinaryPackages]) {
+      if (headerGetEntry(binaryHeaders[numBinaryPackages], RPMTAG_SIZE,
+			 NULL, (void **) &sizep, NULL))
+	totSize += *sizep;
+    } else
+      totSize += UNKNOWN_SIZE;
+    if (rc) {
+      numPackages--;
+      numFailed++;
+    } else if (isSource) {
+      if (binaryHeaders[numBinaryPackages])
+        headerFree(binaryHeaders[numBinaryPackages]);
+      numSourcePackages++;
+    } else
+      numBinaryPackages++;
+  }
+  if (numBinaryPackages) {
+    if (rpmdbOpen(rootdir, &db, mode, 0644)) {
+      for (i=0; i < numBinaryPackages; i++)
+        headerFree(binaryHeaders[i]);
+      g_free(binaryHeaders);
+      g_free(packages);
+      return numPackages;
+    }
+    rpmdep = rpmdepDependencies(db);
+    for (i =0; i < numBinaryPackages; i++)
+      if (installFlags & RPMINSTALL_UPGRADE)
+        rpmdepUpgradePackage(rpmdep, binaryHeaders[i], packages[i]);
+      else
+        rpmdepAddPackage(rpmdep, binaryHeaders[i], packages[i]);
+    if (!(interfaceFlags & INTER_NODEPS)) {
+      if (rpmdepCheck(rpmdep, &conflicts, &numConflicts)) {
+        numFailed = numPackages;
+        message_box(_("Dependency check failed."));
+        stopInstall = 1;
+      }
+      if (!stopInstall && conflicts) {
+        if (!displayDepProblems(conflicts, numConflicts, db, rootdir, NULL)) {
+	  numFailed = numPackages;
+	  stopInstall = 1;
+	}
+	rpmdepFreeConflicts(conflicts, numConflicts);
+      }
+    }
+    if (!(interfaceFlags & INTER_NOORDER))
+      if (rpmdepOrder(rpmdep, (void ***) &packages)) {
+        numFailed = numPackages;
+        stopInstall = 1;
+      }
+    rpmdepDone(rpmdep);
+  } else
+    db = NULL;
+  if (!stopInstall) {
+    /* do the actual install */
+    numFailed += installPackages(rootdir, packages, location, numPackages,
+                                 totSize, installFlags, interfaceFlags, db);
+  }
+  for (i = 0; i < numBinaryPackages; i++)
+    headerFree(binaryHeaders[i]);
+  g_free(binaryHeaders);
+  if (db) rpmdbClose(db);
+  g_free(packages);
+
+  return numFailed;
+}
+
+#else  /* !HAVE_RPM_2_5 */
 
 static gboolean printProblems(rpmProblemSet probs)
 {
   GtkWidget *win, *w, *list;
-  gchar buf[1024];
-  gint i, flags, ret = FALSE;
+  gint i, ret = FALSE;
   GList *items = NULL;
 
   win = gnome_dialog_new(_("Installation Problems"), GNOME_STOCK_BUTTON_YES,
@@ -334,11 +454,11 @@ static gboolean printProblems(rpmProblemSet probs)
 
   for (i = 0; i < probs->numProblems; i++) {
       if (!probs->probs[i].ignoreProblem) {
-	  char *msg = rpmProblemString(probs->probs[i]);
+	  const char *msg = rpmProblemString(probs->probs[i]);
 	  w = gtk_list_item_new_with_label(msg);
 	  gtk_widget_show(w);
 	  items = g_list_append(items, w);
-	  free(msg);
+	  free((void *)msg);
       }
   }
 
@@ -375,7 +495,7 @@ int do_install(gchar *rootdir, GList *pkgs, gchar *location,
   gchar **packages, *pkg_file;
   Header *binaryHeaders;
   rpmdb db;
-  rpmTransactionSet trans = NULL, rpmdep = NULL;
+  rpmTransactionSet rpmdep = NULL;
   rpmProblemSet probs = NULL, finalProbs = NULL;
   GtkWidget *win, *wid;
 
@@ -478,8 +598,6 @@ int do_install(gchar *rootdir, GList *pkgs, gchar *location,
       if (rc > 0) {
 	  if (printProblems(probs)) {
 	      /* ignore all the problems */
-	      rpmProblemSet finalProbes;
-	      rpmProblemSetFilter(probs, -1);
 	      rc = rpmRunTransactions(rpmdep, updateDisp, NULL,
 				      probs, &finalProbs,
 				      transFlags, probFilter);
@@ -504,108 +622,6 @@ int do_install(gchar *rootdir, GList *pkgs, gchar *location,
 
   return numFailed;
 }
-#else  /* !HAVE_RPM_3_0 */
-int do_install(gchar *rootdir, GList *pkgs, gchar *location,
-	       gint installFlags, gint interfaceFlags,
-	       rpmInstallCb cb, void *user_data) {
-  gint mode, rc, i, fd, isSource, stopInstall = 0, pkgc;
-  gint numPackages, numBinaryPackages = 0;
-  gint numSourcePackages = 0, numFailed = 0;
-  gint numConflicts, totSize = 0;
-  gchar **packages, *pkg_file;
-  Header *binaryHeaders;
-  rpmdb db;
-  rpmDependencies rpmdep;
-  struct rpmDependencyConflict *conflicts;
-  gulong *sizep;
-
-  if (installFlags & RPMINSTALL_TEST)
-    mode = O_RDONLY;
-  else
-    mode = O_RDWR | O_CREAT;
-
-  inst_cb = cb;
-  inst_cb_user_data = user_data;
-
-  pkgc = g_list_length(pkgs);
-  packages = g_new(gchar *, pkgc + 1);
-  binaryHeaders = g_new(Header, pkgc + 1);
-  for (numPackages = 0; pkgs != NULL; pkgs = pkgs->next) {
-    pkg_file = pkgs->data;
-    fd = open(pkg_file, O_RDONLY);
-    if (fd < 0) {
-      numFailed++;
-      continue;
-    }
-    packages[numPackages++] = pkg_file;
-    rc = rpmReadPackageHeader(fd, &binaryHeaders[numBinaryPackages],
-                              &isSource, NULL, NULL);
-    close(fd);
-    if (binaryHeaders[numBinaryPackages]) {
-      if (headerGetEntry(binaryHeaders[numBinaryPackages], RPMTAG_SIZE,
-			 NULL, (void **) &sizep, NULL))
-	totSize += *sizep;
-    } else
-      totSize += UNKNOWN_SIZE;
-    if (rc) {
-      numPackages--;
-      numFailed++;
-    } else if (isSource) {
-      if (binaryHeaders[numBinaryPackages])
-        headerFree(binaryHeaders[numBinaryPackages]);
-      numSourcePackages++;
-    } else
-      numBinaryPackages++;
-  }
-  if (numBinaryPackages) {
-    if (rpmdbOpen(rootdir, &db, mode, 0644)) {
-      for (i=0; i < numBinaryPackages; i++)
-        headerFree(binaryHeaders[i]);
-      g_free(binaryHeaders);
-      g_free(packages);
-      return numPackages;
-    }
-    rpmdep = rpmdepDependencies(db);
-    for (i =0; i < numBinaryPackages; i++)
-      if (installFlags & RPMINSTALL_UPGRADE)
-        rpmdepUpgradePackage(rpmdep, binaryHeaders[i], packages[i]);
-      else
-        rpmdepAddPackage(rpmdep, binaryHeaders[i], packages[i]);
-    if (!(interfaceFlags & INTER_NODEPS)) {
-      if (rpmdepCheck(rpmdep, &conflicts, &numConflicts)) {
-        numFailed = numPackages;
-        message_box(_("Dependency check failed."));
-        stopInstall = 1;
-      }
-      if (!stopInstall && conflicts) {
-        if (!displayDepProblems(conflicts, numConflicts, db, rootdir, NULL)) {
-	  numFailed = numPackages;
-	  stopInstall = 1;
-	}
-	rpmdepFreeConflicts(conflicts, numConflicts);
-      }
-    }
-    if (!(interfaceFlags & INTER_NOORDER))
-      if (rpmdepOrder(rpmdep, (void ***) &packages)) {
-        numFailed = numPackages;
-        stopInstall = 1;
-      }
-    rpmdepDone(rpmdep);
-  } else
-    db = NULL;
-  if (!stopInstall) {
-    /* do the actual install */
-    numFailed += installPackages(rootdir, packages, location, numPackages,
-                                 totSize, installFlags, interfaceFlags, db);
-  }
-  for (i = 0; i < numBinaryPackages; i++)
-    headerFree(binaryHeaders[i]);
-  g_free(binaryHeaders);
-  if (db) rpmdbClose(db);
-  g_free(packages);
-
-  return numFailed;
-}
 #endif
 
 static gboolean continue_remove(rpmdb db, GList *indices) {
@@ -628,8 +644,20 @@ static gboolean continue_remove(rpmdb db, GList *indices) {
   gtk_widget_show(w);
   for (tmp = indices; tmp; tmp = tmp->next) {
     guint32 index = GPOINTER_TO_UINT(tmp->data);
-    Header h = rpmdbGetRecord(db, index);
     GtkWidget *item;
+    Header h;
+
+#ifdef	HAVE_RPM_4_0
+    rpmdbMatchIterator mi;
+
+    mi = rpmdbInitIterator(db, RPMDBI_PACKAGES, &index, sizeof(index));
+    h = rpmdbNextIterator(mi);
+    if (h)
+      h = headerLink(h);
+    rpmdbFreeIterator(mi);
+#else
+    h = rpmdbGetRecord(db, index);
+#endif
 
     headerGetEntry(h, RPMTAG_NAME, NULL, (void**)&s1, NULL);
     headerGetEntry(h, RPMTAG_VERSION, NULL, (void**)&s2, NULL);
@@ -657,7 +685,54 @@ static gboolean continue_remove(rpmdb db, GList *indices) {
   ret = gnome_dialog_run_and_close(GNOME_DIALOG(win));
   return !ret;
 }
-#ifdef HAVE_RPM_3_0
+
+#ifdef HAVE_RPM_2_5
+
+gint do_uninstall(gchar *root, GList *indices, gint uninstallFlags,
+		 gint interfaceFlags) {
+  rpmdb db;
+  gint mode, numFailed = 0, numConflicts, stopUninstall = 0;
+  rpmDependencies rpmdep;
+  struct rpmDependencyConflict *conflicts;
+  GList *tmp;
+
+  if (uninstallFlags &RPMUNINSTALL_TEST)
+    mode = O_RDONLY;
+  else
+    mode = O_RDWR | O_EXCL;
+  if (rpmdbOpen(root, &db, mode, 0644)) {
+    return g_list_length(indices);
+  }
+  if (!continue_remove(db, indices)) return 0;
+  if (!(interfaceFlags & INTER_NODEPS)) {
+    rpmdep = rpmdepDependencies(db);
+    for (tmp = indices; tmp != NULL; tmp = tmp->next)
+      rpmdepRemovePackage(rpmdep, GPOINTER_TO_UINT(tmp->data));
+    if (rpmdepCheck(rpmdep, &conflicts, &numConflicts)) {
+      numFailed = g_list_length(indices);
+      stopUninstall = 1;
+    }
+    rpmdepDone(rpmdep);
+    if (!stopUninstall && conflicts) {
+      if (!displayDepProblems(conflicts, numConflicts, db, root, NULL)) {
+	numFailed += g_list_length(indices);
+	stopUninstall = 1;
+      }
+      rpmdepFreeConflicts(conflicts, numConflicts);
+    }
+  }
+  if (!stopUninstall) {
+    for (tmp = indices; tmp != NULL; tmp = tmp->next)
+      if (rpmRemovePackage(root, db, GPOINTER_TO_UINT(tmp->data),
+			   uninstallFlags))
+	numFailed++;
+  }
+  rpmdbClose(db);
+  return numFailed;
+}
+
+#else  /* !HAVE_RPM_2_5 */
+
 gint do_uninstall(gchar *root, GList *indices, gint transFlags,
 		  gint probFilter, gint interfaceFlags) {
   rpmdb db;
@@ -704,64 +779,10 @@ gint do_uninstall(gchar *root, GList *indices, gint transFlags,
   rpmdbClose(db);
   return numFailed;
 }
-#else  /* !HAVE_RPM_3_0 */
-gint do_uninstall(gchar *root, GList *indices, gint uninstallFlags,
-		 gint interfaceFlags) {
-  rpmdb db;
-  gint mode, numFailed = 0, numConflicts, stopUninstall = 0;
-  rpmDependencies rpmdep;
-  struct rpmDependencyConflict *conflicts;
-  GList *tmp;
 
-  if (uninstallFlags &RPMUNINSTALL_TEST)
-    mode = O_RDONLY;
-  else
-    mode = O_RDWR | O_EXCL;
-  if (rpmdbOpen(root, &db, mode, 0644)) {
-    return g_list_length(indices);
-  }
-  if (!continue_remove(db, indices)) return 0;
-  if (!(interfaceFlags & INTER_NODEPS)) {
-    rpmdep = rpmdepDependencies(db);
-    for (tmp = indices; tmp != NULL; tmp = tmp->next)
-      rpmdepRemovePackage(rpmdep, GPOINTER_TO_UINT(tmp->data));
-    if (rpmdepCheck(rpmdep, &conflicts, &numConflicts)) {
-      numFailed = g_list_length(indices);
-      stopUninstall = 1;
-    }
-    rpmdepDone(rpmdep);
-    if (!stopUninstall && conflicts) {
-      if (!displayDepProblems(conflicts, numConflicts, db, root, NULL)) {
-	numFailed += g_list_length(indices);
-	stopUninstall = 1;
-      }
-      rpmdepFreeConflicts(conflicts, numConflicts);
-    }
-  }
-  if (!stopUninstall) {
-    for (tmp = indices; tmp != NULL; tmp = tmp->next)
-      if (rpmRemovePackage(root, db, GPOINTER_TO_UINT(tmp->data),
-			   uninstallFlags))
-	numFailed++;
-  }
-  rpmdbClose(db);
-  return numFailed;
-}
 #endif
-#ifdef HAVE_RPM_3_0
-gint install_one(gchar *root, gchar *file, gchar *location,
-		 gint transFlags, gint probFilter, gint interfaceFlags,
-		 rpmInstallCb cb, void *user_data) {
-  GList *list;
-  gint result;
 
-  list = g_list_append(NULL, file);
-  result = do_install(root, list, location, transFlags, probFilter,
-		      interfaceFlags, cb, user_data);
-  g_list_free(list);
-  return result;
-}
-#else
+#ifdef HAVE_RPM_2_5
 gint install_one(gchar *root, gchar *file, gchar *location,
 		 gint installFlags, gint interfaceFlags,
 		 rpmInstallCb cb, void *user_data) {
@@ -774,18 +795,22 @@ gint install_one(gchar *root, gchar *file, gchar *location,
   g_list_free(list);
   return result;
 }
-#endif
-#ifdef HAVE_RPM_3_0
-gint uninstall_one(gchar *root, guint index, gint transFlags,
-		   gint probFilter, gint interfaceFlags) {
+#else
+gint install_one(gchar *root, gchar *file, gchar *location,
+		 gint transFlags, gint probFilter, gint interfaceFlags,
+		 rpmInstallCb cb, void *user_data) {
   GList *list;
   gint result;
-  list = g_list_append(NULL, GUINT_TO_POINTER(index));
-  result = do_uninstall(root, list, transFlags, probFilter, interfaceFlags);
+
+  list = g_list_append(NULL, file);
+  result = do_install(root, list, location, transFlags, probFilter,
+		      interfaceFlags, cb, user_data);
   g_list_free(list);
   return result;
 }
-#else
+#endif
+
+#ifdef HAVE_RPM_2_5
 gint uninstall_one(gchar *root, guint index, gint uninstallFlags,
 		  gint interfaceFlags) {
   GList *list;
@@ -795,4 +820,14 @@ gint uninstall_one(gchar *root, guint index, gint uninstallFlags,
   g_list_free(list);
   return result;
 }
+#else
+gint uninstall_one(gchar *root, guint index, gint transFlags,
+		   gint probFilter, gint interfaceFlags) {
+  GList *list;
+  gint result;
+  list = g_list_append(NULL, GUINT_TO_POINTER(index));
+  result = do_uninstall(root, list, transFlags, probFilter, interfaceFlags);
+  g_list_free(list);
+  return result;
+}
 #endif
