@@ -1,5 +1,5 @@
 /* GnoRPM - A GNOME front end for the Red Hat Package Manager (RPM)
- * Copyright (C) 1998-1999  James Henstridge
+ * Copyright (C) 1998-1999 James Henstridge
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -41,6 +41,7 @@
 #endif
 
 GnomeApp *gnorpm_app = NULL;
+GtkWidget *statusbar = NULL;
 
 enum { TARGET_URI_LIST };
 static GtkTargetEntry drop_types[] = {
@@ -61,55 +62,76 @@ static void mainwin_drop_cb(GnomeApp *app, GdkDragContext *context,
 static void mainwin_context_menu(RpmPackageList *pl, GdkEventButton *event,
 				 guint index);
 
-#ifdef HAVE_RPM_3_0
-static guint interfaceFlags = 0, transFlags = 0, probFilter = 0;
-#else
+#ifdef HAVE_RPM_2_5
 static guint interfaceFlags = 0, installFlags = 0, uninstallFlags = 0;
+#else
+static guint interfaceFlags = 0, transFlags = 0, probFilter = 0;
 #endif
 
 #ifdef WITH_RPMFIND
+
+extern int downloadAbort;
+
+static void gnorpm_download_abort(gpointer crap)
+{
+  downloadAbort=1;
+}
+
+static void mainwin_closed(gpointer crap) {
+	downloadAbort = 1;
+	statusbar = NULL;
+}
+
 static void gnorpm_download_start(char *url) {
   char *msg = g_strconcat("Downloading ", url, " ...", NULL);
   GnomeAppProgressKey key = gnome_app_progress_manual(gnorpm_app, msg,
-						      NULL, NULL);
+						      gnorpm_download_abort, NULL);
   GSList *list;
 
   list = gtk_object_get_data(GTK_OBJECT(gnorpm_app), "download_key");
   list = g_slist_prepend(list, key);
   gtk_object_set_data(GTK_OBJECT(gnorpm_app), "download_key", list);
   g_free(msg);
+  while (gtk_events_pending())
+    gtk_main_iteration();
 }
 
 static void gnorpm_download_progress(unsigned long done, unsigned long total) {
-  GSList *list = gtk_object_get_data(GTK_OBJECT(gnorpm_app), "download_key");
+  GSList *list;
   GnomeAppProgressKey key;
 
-  g_return_if_fail(list != NULL);
-  key = (GnomeAppProgressKey) list->data;
+  if(!downloadAbort && !GTK_OBJECT_DESTROYED(GTK_OBJECT(gnorpm_app))) {
+    list = gtk_object_get_data(GTK_OBJECT(gnorpm_app), "download_key");
+    g_return_if_fail(list != NULL);
+    key = (GnomeAppProgressKey) list->data;
 
-  if (total == 0) total = 1;
-  gnome_app_set_progress(key, ((gdouble)done)/total);
+    if (total == 0) total = 1;
+    gnome_app_set_progress(key, ((gdouble)done)/total);
+  }
 
   while (gtk_events_pending())
     gtk_main_iteration();
 }
 
 static void gnorpm_download_done(void) {
-  GSList *list = gtk_object_get_data(GTK_OBJECT(gnorpm_app), "download_key");
+  GSList *list;
   GnomeAppProgressKey key;
 
-  g_return_if_fail(list != NULL);
-  key = (GnomeAppProgressKey) list->data;
-  list = g_slist_remove(list, key);
+  if(!downloadAbort && !GTK_OBJECT_DESTROYED(GTK_OBJECT(gnorpm_app))) {
+    list = gtk_object_get_data(GTK_OBJECT(gnorpm_app), "download_key");
+    g_return_if_fail(list != NULL);
+    key = (GnomeAppProgressKey) list->data;
+    list = g_slist_remove(list, key);
 
-  gnome_app_progress_done(key);
-  gnome_appbar_refresh(GNOME_APPBAR(gnorpm_app->statusbar));
-  gtk_object_set_data(GTK_OBJECT(gnorpm_app), "download_key", list);
+    gnome_app_progress_done(key);
+    gnome_appbar_refresh(GNOME_APPBAR(gnorpm_app->statusbar));
+    gtk_object_set_data(GTK_OBJECT(gnorpm_app), "download_key", list);
+  }
 }
 #endif
 
 GtkWidget *create_main(char *app_id, DBHandle *hdl) {
-  GtkWidget *app, *pl, *status;
+  GtkWidget *app, *pl;
 
   app = gnome_app_new(app_id, _("Gnome RPM"));
   gnorpm_app = GNOME_APP(app);
@@ -123,13 +145,13 @@ GtkWidget *create_main(char *app_id, DBHandle *hdl) {
   gtk_signal_connect(GTK_OBJECT(pl), "context_menu",
 		     GTK_SIGNAL_FUNC(mainwin_context_menu), NULL);
 
-  status = gnome_appbar_new(TRUE, TRUE, GNOME_PREFERENCES_USER);
-  gnome_app_set_statusbar(GNOME_APP(app), status);
-  gtk_widget_show(status);
+  statusbar = gnome_appbar_new(TRUE, TRUE, GNOME_PREFERENCES_USER);
+  gnome_app_set_statusbar(GNOME_APP(app), statusbar);
+  gtk_widget_show(statusbar);
 
-  gnome_appbar_push(GNOME_APPBAR(status), _("Packages Selected: 0"));
+  gnome_appbar_push(GNOME_APPBAR(statusbar), _("Packages Selected: 0"));
   gtk_signal_connect(GTK_OBJECT(pl), "selection_changed",
-		     GTK_SIGNAL_FUNC(update_label), status);
+		     GTK_SIGNAL_FUNC(update_label), statusbar);
 
   create_menus(app, pl);
   set_icon(app);
@@ -138,13 +160,14 @@ GtkWidget *create_main(char *app_id, DBHandle *hdl) {
   url_set_callbacks(gnorpm_download_start,
 		    gnorpm_download_progress,
 		    gnorpm_download_done);
+  set_complain_func(statusbar_msg);
 #endif
 
   /* get the saved properties */
-#ifdef HAVE_RPM_3_0
-  rpm_props_box_get_flags(&interfaceFlags, &transFlags, &probFilter);
-#else    
+#ifdef HAVE_RPM_2_5
   rpm_props_box_get_flags(&interfaceFlags, &installFlags, &uninstallFlags);
+#else    
+  rpm_props_box_get_flags(&interfaceFlags, &transFlags, &probFilter);
 #endif
   
   if (gnome_config_get_bool("/gnorpm/Layout/asIcons=true"))
@@ -190,7 +213,6 @@ static void rpm_show_web_find(GtkWidget *mi, RpmPackageList *pi);
 #endif
 static void rpm_clear_selection(GtkWidget *mi, RpmPackageList *pl);
 static void rpm_install_pkgs(GtkWidget *mi, RpmPackageList *pl);
-static void rpm_upgrade_pkgs(GtkWidget *mi, RpmPackageList *pl);
 static void rpm_uninstall_pkgs(GtkWidget *mi, RpmPackageList *pl);
 static void rpm_verify_pkgs(GtkWidget *mi, RpmPackageList *pl);
 static void rpm_query_pkgs(GtkWidget *mi, RpmPackageList *pl);
@@ -297,11 +319,18 @@ static void rpm_clear_selection(GtkWidget *mi, RpmPackageList *pl) {
 }
 
 static void rpm_uninstall_pkgs(GtkWidget *mi, RpmPackageList *pl) {
-#ifdef HAVE_RPM_3_0
+  if(geteuid())
+  {
+    GtkWidget *d=gnome_error_dialog(_("You need to be the superuser to uninstall packages."));
+    gnome_dialog_run_and_close(GNOME_DIALOG(d));
+    return;
+  }
+
+#ifdef HAVE_RPM_2_5
+  do_uninstall(pl->hdl->root, pl->selection, uninstallFlags, interfaceFlags);
+#else
   do_uninstall(pl->hdl->root, pl->selection, transFlags, probFilter,
 	       interfaceFlags);
-#else
-  do_uninstall(pl->hdl->root, pl->selection, uninstallFlags, interfaceFlags);
 #endif
   /* update the package list display */
   rpm_package_list_clear_selection(pl);
@@ -309,10 +338,22 @@ static void rpm_uninstall_pkgs(GtkWidget *mi, RpmPackageList *pl) {
 }
 
 static void rpm_verify_pkgs(GtkWidget *mi, RpmPackageList *pl) {
+  if(geteuid())
+  {
+    GtkWidget *d=gnome_error_dialog(_("You need to be the superuser to verify packages."));
+    gnome_dialog_run_and_close(GNOME_DIALOG(d));
+    return;
+  }
   verify_packages(pl->hdl, pl->selection, 0);
 }
 
 static void verify_func(RpmQuery *info, gpointer data) {
+  if(geteuid())
+  {
+    GtkWidget *d=gnome_error_dialog(_("You need to be the superuser to verify packages."));
+    gnome_dialog_run_and_close(GNOME_DIALOG(d));
+    return;
+  }
   verify_one(info->hdl, info->index, 0);
 }
 
@@ -327,12 +368,12 @@ static void uninstall_func(RpmQuery *info, gpointer data) {
   char buf[512];
   RpmPackageList *pl = RPM_PACKAGE_LIST(data);
 
-#ifdef HAVE_RPM_3_0
-  failed = uninstall_one(info->hdl->root, info->index, transFlags,
-			 probFilter, interfaceFlags);
-#else
+#ifdef HAVE_RPM_2_5
   failed = uninstall_one(info->hdl->root, info->index, uninstallFlags,
 			 interfaceFlags);
+#else
+  failed = uninstall_one(info->hdl->root, info->index, transFlags,
+			 probFilter, interfaceFlags);
 #endif
   if (failed) {
     g_snprintf(buf, 511, _("%d packages couldn't be uninstalled"), failed);
@@ -344,10 +385,10 @@ static void uninstall_func(RpmQuery *info, gpointer data) {
   rpm_package_list_unselect(pl, info->index);
 
   /* remove the package from the query window */
-#ifdef HAVE_RPM_3_0
-  if (!(transFlags & RPMTRANS_FLAG_TEST)) {
+#ifdef HAVE_RPM_2_5
+  if (!(uninstallFlags & RPMUNINSTALL_TEST)) {
 #else
-    if (!(uninstallFlags & RPMUNINSTALL_TEST)) {
+  if (!(transFlags & RPMTRANS_FLAG_TEST)) {
 #endif
     GtkWidget *parent = GTK_WIDGET(info)->parent;
 
@@ -400,11 +441,11 @@ static void rpm_find_uninstall_cb(RpmFindDialog *dlg, GList *indices,
   GList *tmp;
 
   if (indices == NULL) return;
-#ifdef HAVE_RPM_3_0
+#ifdef HAVE_RPM_2_5
+  failed = do_uninstall(pl->hdl->root, indices, uninstallFlags,interfaceFlags);
+#else
   failed = do_uninstall(pl->hdl->root, indices, transFlags, probFilter,
 			interfaceFlags);
-#else
-  failed = do_uninstall(pl->hdl->root, indices, uninstallFlags,interfaceFlags);
 #endif
   if (failed) {
     g_snprintf(buf, 511, _("%d packages couldn't be uninstalled"), failed);
@@ -419,6 +460,12 @@ static void rpm_find_uninstall_cb(RpmFindDialog *dlg, GList *indices,
 static void rpm_find_verify_cb(RpmFindDialog *dlg, GList *indices,
 			RpmPackageList *pl) {
   if (indices == NULL) return;
+  if(geteuid())
+  {
+    GtkWidget *d=gnome_error_dialog(_("You need to be the superuser to verify packages."));
+    gnome_dialog_run_and_close(GNOME_DIALOG(d));
+    return;
+  }
   verify_packages(pl->hdl, indices, 0);
 }
 
@@ -442,11 +489,11 @@ static void gnorpm_update_tree(gchar *pkgName, gchar *groupName, void *data) {
 }
 
 static void install_func(RpmQuery *info, RpmPackageList *pl) {
-#ifdef HAVE_RPM_3_0
-  if (install_one(info->hdl->root, info->fname, NULL, transFlags, probFilter,
+#ifdef HAVE_RPM_2_5
+  if (install_one(info->hdl->root, info->fname, NULL, installFlags,
 		  interfaceFlags, gnorpm_update_tree, pl)) {
 #else
-  if (install_one(info->hdl->root, info->fname, NULL, installFlags,
+  if (install_one(info->hdl->root, info->fname, NULL, transFlags, probFilter,
 		  interfaceFlags, gnorpm_update_tree, pl)) {
 #endif
     gchar buf[512];
@@ -456,13 +503,13 @@ static void install_func(RpmQuery *info, RpmPackageList *pl) {
   rpm_package_list_update_pane(pl);
 }
 static void upgrade_func(RpmQuery *info, RpmPackageList *pl) {
-#ifdef HAVE_RPM_3_0
+#ifdef HAVE_RPM_2_5
   if (install_one(info->hdl->root, info->fname, NULL,
-		  transFlags, probFilter, interfaceFlags | INTER_UPGRADE,
+		  installFlags | RPMINSTALL_UPGRADE, interfaceFlags,
 		  gnorpm_update_tree, pl)) {
 #else
   if (install_one(info->hdl->root, info->fname, NULL,
-		  installFlags | RPMINSTALL_UPGRADE, interfaceFlags,
+		  transFlags, probFilter, interfaceFlags | INTER_UPGRADE,
 		  gnorpm_update_tree, pl)) {
 #endif      
     gchar buf[512];
@@ -500,12 +547,12 @@ static void rpm_install_pkgs_cb(RpmInstallDialog *dlg, GList *files,
     message_box(_("No packages selected"));
     return;
   }
-#ifdef HAVE_RPM_3_0
-  res = do_install(dlg->hdl->root, files, NULL, transFlags, probFilter,
-		   interfaceFlags, gnorpm_update_tree, pl);
-#else
+#ifdef HAVE_RPM_2_5
   res = do_install(dlg->hdl->root, files, NULL, installFlags, interfaceFlags,
 		   gnorpm_update_tree, pl);
+#else
+  res = do_install(dlg->hdl->root, files, NULL, transFlags, probFilter,
+		   interfaceFlags, gnorpm_update_tree, pl);
 #endif
   if (res) {
     gchar buf[512];
@@ -523,13 +570,13 @@ static void rpm_upgrade_pkgs_cb(RpmInstallDialog *dlg, GList *files,
     message_box(_("No packages selected"));
     return;
   }
-#ifdef HAVE_RPM_3_0
+#ifdef HAVE_RPM_2_5
   res = do_install(dlg->hdl->root, files, NULL,
-		   transFlags, probFilter, interfaceFlags | INTER_UPGRADE,
+		   installFlags | RPMINSTALL_UPGRADE, interfaceFlags,
 		   gnorpm_update_tree, pl);
 #else
   res = do_install(dlg->hdl->root, files, NULL,
-		   installFlags | RPMINSTALL_UPGRADE, interfaceFlags,
+		   transFlags, probFilter, interfaceFlags | INTER_UPGRADE,
 		   gnorpm_update_tree, pl);
 #endif
 
@@ -552,6 +599,14 @@ static void rpm_check_sigs_cb(RpmInstallDialog *dlg, GList *files) {
 
 static void rpm_install_pkgs(GtkWidget *mi, RpmPackageList *pl) {
   GtkWidget *win;
+
+  if(geteuid())
+  {
+    GtkWidget *d=gnome_error_dialog(_("You need to be the superuser to install packages."));
+    gnome_dialog_run_and_close(GNOME_DIALOG(d));
+    return;
+  }
+
   win = rpm_install_dialog_new(pl->hdl);
   rpm_install_dialog_add_default_packages(RPM_INSTALL_DIALOG(win));
   gtk_window_set_title(GTK_WINDOW(win), _("Install"));
@@ -578,7 +633,7 @@ static void mainwin_drop_cb(GnomeApp *app, GdkDragContext *context,
     win = rpm_install_dialog_new(pl->hdl);
     gtk_window_set_title(GTK_WINDOW(win), _("Install"));
     gtk_signal_connect(GTK_OBJECT(win), "query",
-		       GTK_SIGNAL_FUNC(rpm_instpkgs_query), pl->hdl);
+		       GTK_SIGNAL_FUNC(rpm_instpkgs_query), pl);
     gtk_signal_connect(GTK_OBJECT(win), "install",
 		       GTK_SIGNAL_FUNC(rpm_install_pkgs_cb), pl);
     gtk_signal_connect(GTK_OBJECT(win), "checksig",
@@ -602,6 +657,8 @@ static void rpm_show_web_find(GtkWidget *mi, RpmPackageList *pl) {
 			    GTK_SIGNAL_FUNC(gtk_widget_destroy),
 			    GTK_OBJECT(win));
   gtk_window_set_title(GTK_WINDOW(win), _("Rpmfind"));
+  gtk_object_weakref(GTK_OBJECT(mi), (GtkDestroyNotify) mainwin_closed,
+ 		     NULL);
   rpm_web_find_set_callbacks(RPM_WEB_FIND(webfind),
 			     GTK_SIGNAL_FUNC(rpm_install_pkgs_cb),
 			     GTK_SIGNAL_FUNC(rpm_upgrade_pkgs_cb),
@@ -625,14 +682,14 @@ static void rpm_preferences_apply_cb(RpmPropsBox *box, gint page_num,
 				     RpmPackageList *pl) {
   switch (page_num) {
   case RPM_PROPS_FLAGS:
-#ifdef HAVE_RPM_3_0
-    interfaceFlags = box->interfaceFlags;
-    transFlags     = box->transFlags;
-    probFilter     = box->probFilter;
-#else
+#ifdef HAVE_RPM_2_5
     interfaceFlags = box->interfaceFlags;
     installFlags   = box->installFlags;
     uninstallFlags = box->uninstallFlags;
+#else
+    interfaceFlags = box->interfaceFlags;
+    transFlags     = box->transFlags;
+    probFilter     = box->probFilter;
 #endif
     break;
   case RPM_PROPS_PKGLIST:
@@ -666,7 +723,6 @@ static void rpm_preferences_cb(GtkWidget *mi, RpmPackageList *pl) {
 }
 
 static void context_query(GtkWidget *mi, gpointer index_p) {
-  guint index = GPOINTER_TO_UINT(index_p);
   RpmPackageList *pl = gtk_object_get_data(GTK_OBJECT(mi), "package-list");
   GList item;
   GtkWidget *win;
@@ -684,16 +740,15 @@ static void context_query(GtkWidget *mi, gpointer index_p) {
 
 static void context_uninstall(GtkWidget *mi, gpointer index_p) {
   gint failed;
-  char buf[512];
   guint index = GPOINTER_TO_UINT(index_p);
 
   RpmPackageList *pl = gtk_object_get_data(GTK_OBJECT(mi), "package-list");
 
-#ifdef HAVE_RPM_3_0
+#ifdef HAVE_RPM_2_5
+  failed = uninstall_one(pl->hdl->root, index, uninstallFlags, interfaceFlags);
+#else
   failed = uninstall_one(pl->hdl->root, index, transFlags, probFilter,
 			 interfaceFlags);
-#else
-  failed = uninstall_one(pl->hdl->root, index, uninstallFlags, interfaceFlags);
 #endif
   if (failed) {
     char buf[512];
@@ -725,18 +780,38 @@ static GnomeUIInfo context_menu[] = {
     GNOMEUIINFO_END
 };
 
+static GnomeUIInfo context_menu_user[] = {
+    GNOMEUIINFO_ITEM_STOCK(N_("_Query"),
+			N_("Get information about the selected packages"),
+			context_query, RPM_STOCK_MENU_QUERY),
+    GNOMEUIINFO_ITEM_STOCK(N_("_Uninstall"),
+			N_("Uninstall the selected packages"),
+			context_uninstall, RPM_STOCK_MENU_UNINSTALL),
+    GNOMEUIINFO_END
+};
+
 static void mainwin_context_menu(RpmPackageList *pl, GdkEventButton *event,
 				 guint index) {
   GtkWidget *menu;
 
   menu = gtk_object_get_data(GTK_OBJECT(pl), "context-menu");
   if (!menu) {
-    menu = gnome_popup_menu_new(context_menu);
-    gnome_app_install_menu_hints(gnorpm_app, context_menu);
-    gtk_object_set_data(GTK_OBJECT(context_menu[0].widget), "package-list",pl);
-    gtk_object_set_data(GTK_OBJECT(context_menu[1].widget), "package-list",pl);
-    gtk_object_set_data(GTK_OBJECT(context_menu[2].widget), "package-list",pl);
+    if(geteuid())
+    {
+      menu = gnome_popup_menu_new(context_menu_user);
+      gnome_app_install_menu_hints(gnorpm_app, context_menu_user);
+      gtk_object_set_data(GTK_OBJECT(context_menu_user[0].widget), "package-list",pl);
+      gtk_object_set_data(GTK_OBJECT(context_menu_user[1].widget), "package-list",pl);
+    }
+    else
+    {
+      menu = gnome_popup_menu_new(context_menu);
+      gnome_app_install_menu_hints(gnorpm_app, context_menu);
+      gtk_object_set_data(GTK_OBJECT(context_menu[0].widget), "package-list",pl);
+      gtk_object_set_data(GTK_OBJECT(context_menu[1].widget), "package-list",pl);
+      gtk_object_set_data(GTK_OBJECT(context_menu[2].widget), "package-list",pl);
+    }
     gtk_object_set_data(GTK_OBJECT(pl), "context-menu", menu);
   }
   gnome_popup_menu_do_popup(menu, NULL, NULL, event, GUINT_TO_POINTER(index));
-}
+}
\ No newline at end of file
