@@ -27,6 +27,7 @@
 #include "rpm-icon.xpm"
 
 extern GnomeApp *gnorpm_app;
+extern GtkWidget *statusbar;
 
 void message_box(char *msg) {
   GtkWidget *win;
@@ -66,3 +67,20 @@ void set_icon(GtkWidget *w) {
   icon = gdk_pixmap_create_from_xpm_d(win, &mask, NULL, rpm_icon);
   gdk_window_set_icon(win, NULL, icon, mask);
 }
+
+void statusbar_msg(char *txt, ...)
+{
+	char buf[512];
+	va_list ap;
+
+	va_start(ap, txt);
+	vsnprintf(buf,512,txt,ap);
+	va_end(ap);
+
+	if(statusbar)
+	{
+		gnome_appbar_pop(GNOME_APPBAR(statusbar));
+  		gnome_appbar_push(GNOME_APPBAR(statusbar), buf);
+	}
+	else g_warning(buf);
+}
