@@ -122,7 +122,7 @@ static void rpm_find_dialog_init(RpmFindDialog *self) {
 
   for (i = 0; i < sizeof(entries)/sizeof(GtkItemFactoryEntry); i++)
     entries[i].path = _(entries[i].path);
-  gtk_window_set_title(GTK_WINDOW(self), N_("Find Packages"));
+  gtk_window_set_title(GTK_WINDOW(self), _("Find Packages"));
   set_icon(GTK_WIDGET(self));
   
   vbox = gtk_vbox_new(FALSE, 5);
@@ -230,15 +230,56 @@ static void menu_callback(RpmFindDialog *dlg, guint find_type,
 static void rpm_find_dialog_find(RpmFindDialog *self) {
   GtkWidget *item;
   GList *items = NULL;
-  Header h;
-  gchar buf[512], *s1, *s2, *s3, *search;
+#ifdef HAVE_RPM_4_0
+  rpmdbMatchIterator mi;
+  gint rpmtag = -1;
+#else
   dbiIndexSet matches;
-  gint res = 1, i, index;
+  gchar buf[512], *s1, *s2, *s3;
+  gint res = 1, i;
+#endif
+  gchar *search;
+  Header h;
 
   g_return_if_fail(self->hdl != NULL);
+  db_handle_db_up(self->hdl);
   g_return_if_fail(self->hdl->db != NULL);
   
   search = gtk_entry_get_text(GTK_ENTRY(self->entry));
+
+#ifdef HAVE_RPM_4_0
+  switch (self->find_type) {
+  case FIND_BY_FILE:
+    rpmtag = RPMTAG_BASENAMES;         break;
+  case FIND_BY_GROUP:
+    rpmtag = RPMTAG_GROUP;             break;
+  case FIND_BY_PROVIDES:
+    rpmtag = RPMTAG_PROVIDENAME;       break;
+  case FIND_BY_REQUIREDBY:
+    rpmtag = RPMTAG_REQUIRENAME;       break;
+  case FIND_BY_CONFLICTS:
+    rpmtag = RPMTAG_CONFLICTNAME;      break;
+  case FIND_BY_LABEL:
+    rpmtag = RPMDBI_LABEL;             break;
+  }
+  gtk_list_clear_items(GTK_LIST(self->list), 0, -1);
+  if (rpmtag < 0) return;
+  mi = rpmdbInitIterator(self->hdl->db, rpmtag, search, 0);
+  while ((h = rpmdbNextIterator(mi)) != NULL) {
+    const gchar *s1, *s2, *s3;
+    gchar buf[512];
+    headerNVR(h, &s1, &s2, &s3);
+    g_snprintf(buf, sizeof(buf)-1, "%s-%s-%s", s1, s2, s3);
+    item = gtk_list_item_new_with_label(buf);
+    gtk_object_set_user_data(GTK_OBJECT(item), GINT_TO_POINTER(index));
+    items = g_list_prepend(items, item);
+    gtk_widget_show(item);
+  }
+  gtk_list_append_items(GTK_LIST(self->list), items);
+  db_handle_db_down(self->hdl);
+
+#else  /* !HAVE_RPM_4_0 */
+
   switch (self->find_type) {
   case FIND_BY_FILE:
     res = rpmdbFindByFile(self->hdl->db, search, &matches);      break;
@@ -260,6 +301,7 @@ static void rpm_find_dialog_find(RpmFindDialog *self) {
   gtk_list_clear_items(GTK_LIST(self->list), 0, -1);
   if (res) return;
   for (i = matches.count - 1; i >= 0; i--) {
+    gint index;
     index = matches.recs[i].recOffset;
     if (!index) continue;  /* some queries return this index causing a segv */
     h = rpmdbGetRecord(self->hdl->db, index);
@@ -275,6 +317,8 @@ static void rpm_find_dialog_find(RpmFindDialog *self) {
   }
   gtk_list_append_items(GTK_LIST(self->list), items);
   dbiFreeIndexRecord(matches);
+  db_handle_db_down(self->hdl);
+#endif /* HAVE_RPM_4_0 */
 }
 
 static void rpm_find_dialog_query(RpmFindDialog *self) {
