@@ -38,7 +38,6 @@ static void rpm_package_list_class_init(RpmPackageListClass *klass);
 static void rpm_package_list_init(RpmPackageList *list);
 gint rpm_package_list_set_db(RpmPackageList *list, DBHandle *db);
 
-static void rpm_package_list_chk_scroll(GtkAdjustment *adj, GtkWidget *scroll);
 static void rpm_package_list_destroy(GtkObject *obj);
 static void rpm_package_list_update_clist(RpmPackageList *list,
 					  gchar *group_name);
@@ -199,7 +198,11 @@ GtkWidget *rpm_package_list_new(DBHandle *hdl) {
 gint rpm_package_list_set_db(RpmPackageList *self, DBHandle *hdl) {
   GtkPathTree *tree;
   Header h;
+#ifdef	HAVE_RPM_4_0
+  rpmdbMatchIterator mi;
+#else
   gint i;
+#endif
   gchar *group;
 
   self->hdl = hdl;
@@ -207,6 +210,15 @@ gint rpm_package_list_set_db(RpmPackageList *self, DBHandle *hdl) {
 
   tree = GTK_PATH_TREE(self->tree);
   gtk_path_tree_freeze(tree);
+
+#ifdef	HAVE_RPM_4_0
+  mi = rpmdbInitIterator(self->hdl->db, RPMDBI_PACKAGES, NULL, 0);
+  while ((h = rpmdbNextIterator(mi)) != NULL) {
+    if (headerGetEntry(h, RPMTAG_GROUP, NULL, (void **) &group, NULL))
+      gtk_path_tree_add_path(tree, group);
+  }
+  rpmdbFreeIterator(mi);
+#else	/* !HAVE_RPM_4_0 */
   for (i = rpmdbFirstRecNum(self->hdl->db); i > 0;
        i = rpmdbNextRecNum(self->hdl->db, i)) {
     h = rpmdbGetRecord(self->hdl->db, i);
@@ -216,22 +228,14 @@ gint rpm_package_list_set_db(RpmPackageList *self, DBHandle *hdl) {
       headerFree(h);
     }
   }
+#endif	/* !HAVE_RPM_4_0 */
+
   gtk_path_tree_thaw(tree);
 
   db_handle_db_down(self->hdl);
   return 0;
 }
 
-static void rpm_package_list_chk_scroll(GtkAdjustment *adj, GtkWidget *scroll){
-  if (adj->upper - adj->lower <= adj->page_size) {
-    if (GTK_WIDGET_VISIBLE(scroll))
-      gtk_widget_hide(scroll);
-  } else {
-    if (!GTK_WIDGET_VISIBLE(scroll))
-      gtk_widget_show(scroll);
-  }
-}
-
 void rpm_package_list_clear_selection(RpmPackageList *self) {
   /* clear out selection */
   g_list_free(self->selection);
@@ -293,20 +297,25 @@ static GdkImlibImage *rpm_package_list_get_icon(RpmPackageList *self,
   gchar *data;
   gint fd, len;
   GdkImlibImage *ret;
+  char *workpath;
   if (headerGetEntry(h, RPMTAG_GIF, NULL, (void **)&data, &len)) {
-    fd = open("/tmp/gnorpm.gif", O_RDWR | O_CREAT, 0666);
+    workpath = gnome_util_home_file("gnorpm.d/gnorpm.gif");
+    fd = open(workpath, O_RDWR | O_CREAT, 0666);
     write(fd, data, len);
     close(fd);
-    ret = gdk_imlib_load_image("/tmp/gnorpm.gif");
+    ret = gdk_imlib_load_image(workpath);
     gdk_imlib_changed_image(ret); /* prevent caching */
-    unlink("/tmp/gnorpm.gif");
+    unlink(workpath);
+    g_free(workpath);
   } else if (headerGetEntry(h, RPMTAG_XPM, NULL, (void**)&data, &len)) {
-    fd = open("/tmp/gnorpm.xpm", O_RDWR | O_CREAT, 0666);
+    workpath = gnome_util_home_file("gnorpm.d/gnorpm.xpm");
+    fd = open(workpath, O_RDWR | O_CREAT, 0666);
     write(fd, data, len);
     close(fd);
-    ret = gdk_imlib_load_image("/tmp/gnorpm.xpm");
+    ret = gdk_imlib_load_image(workpath);
     gdk_imlib_changed_image(ret); /* prevent caching */
-    unlink("/tmp/gnorpm.xpm");
+    unlink(workpath);
+    g_free(workpath);
   } else
     /* fallback ... */
     ret = gdk_imlib_create_image_from_xpm_data(gnome_package_xpm);
@@ -322,8 +331,13 @@ static void rpm_package_list_update_clist(RpmPackageList *self,
 
 void rpm_package_list_update_pane(RpmPackageList *self) {
   gchar *row[4], text[50], *tmp;
+#ifdef	HAVE_RPM_4_0
+  rpmdbMatchIterator mi;
+#else
   dbiIndexSet matches;
-  gint i, row_id, icon_id;
+  gint i;
+#endif
+  gint row_id, icon_id;
   Header h;
   GtkCList *clist;
   GnomeIconList *ilist;
@@ -335,13 +349,19 @@ void rpm_package_list_update_pane(RpmPackageList *self) {
   if (!self->cur_group) {
     gtk_clist_clear(clist);
     gnome_icon_list_clear(ilist);
+    g_list_free(self->selection);
+    self->selection = NULL;
     return;
   }
 
   db_handle_db_up(self->hdl);
 
-  if (rpmdbFindByGroup(self->hdl->db, self->cur_group, &matches)) {
+#ifdef	HAVE_RPM_4_0
+  mi = rpmdbInitIterator(self->hdl->db, RPMTAG_GROUP, self->cur_group, 0);
+
+  if (rpmdbGetIteratorCount(mi) == 0) {
     /* no matches ... */
+    rpmdbFreeIterator(mi);
     gtk_clist_clear(clist);
     gnome_icon_list_clear(ilist);
     db_handle_db_down(self->hdl);
@@ -352,6 +372,55 @@ void rpm_package_list_update_pane(RpmPackageList *self) {
   gtk_clist_clear(clist);
   gnome_icon_list_freeze(ilist);
   gnome_icon_list_clear(ilist);
+
+  while ((h = rpmdbNextIterator(mi)) != NULL) {
+    headerGetEntry(h, RPMTAG_NAME,    NULL, (void**)&row[0], NULL);
+    headerGetEntry(h, RPMTAG_VERSION, NULL, (void**)&row[1], NULL);
+    headerGetEntry(h, RPMTAG_RELEASE, NULL, (void**)&row[2], NULL);
+    headerGetEntry(h, RPMTAG_SUMMARY, NULL, (void**)&row[3], NULL);
+    g_snprintf(text, 49, "%s\n%s-%s", row[0], row[1], row[2]);
+
+    for (row_id = 0; row_id < clist->rows; row_id++) {
+      gtk_clist_get_text(clist, row_id, 0, &tmp);
+      if (strcmp(tmp, row[0]) > 0)
+	break;
+    }
+    gtk_clist_insert(clist, row_id, row);
+    gtk_clist_set_row_data(clist, row_id,
+			   GUINT_TO_POINTER(rpmdbGetIteratorOffset(mi)));
+    im = rpm_package_list_get_icon(self, h);
+    icon_id = gnome_icon_list_append_imlib(ilist, im, text);
+    gnome_icon_list_set_icon_data(ilist, icon_id,
+				  GUINT_TO_POINTER(rpmdbGetIteratorOffset(mi)));
+
+    if (g_list_find(self->selection,
+		    GUINT_TO_POINTER(rpmdbGetIteratorOffset(mi)))) {
+      gtk_clist_select_row(clist, row_id, -1);
+      gnome_icon_list_select_icon(ilist, icon_id);
+    }
+  }
+
+  rpmdbFreeIterator(mi);
+
+#else	/* !HAVE_RPM_4_0 */
+
+  if (rpmdbFindByGroup(self->hdl->db, self->cur_group, &matches)) {
+    /* no matches ... */
+    gtk_clist_clear(clist);
+    gnome_icon_list_clear(ilist);
+    g_list_free(self->selection);
+    self->selection = NULL;
+    db_handle_db_down(self->hdl);
+    return;
+  }
+
+  gtk_clist_freeze(clist);
+  gtk_clist_clear(clist);
+  gnome_icon_list_freeze(ilist);
+  gnome_icon_list_clear(ilist);
+  g_list_free(self->selection);
+  self->selection = NULL;
+  
   for (i = 0; i < matches.count; i++) {
     h = rpmdbGetRecord(self->hdl->db, matches.recs[i].recOffset);
     headerGetEntry(h, RPMTAG_NAME,    NULL, (void**)&row[0], NULL);
@@ -381,6 +450,8 @@ void rpm_package_list_update_pane(RpmPackageList *self) {
     headerFree(h);
   }
   dbiFreeIndexRecord(matches);
+#endif	/* !HAVE_RPM_4_0 */
+
   gtk_clist_thaw(clist);
   gnome_icon_list_thaw(ilist);
 
@@ -393,7 +464,7 @@ static void rpm_package_list_select_row(RpmPackageList *self, gint row,
 
   /* column is set < 0 only when we redo selection of rows when we change
    * package group. */
-  if (event == NULL) return;
+  if (self->mode != RPM_PACKAGE_CLIST) return;
   offset =GPOINTER_TO_UINT(gtk_clist_get_row_data(GTK_CLIST(self->clist),row));
   if (g_list_find(self->selection, GUINT_TO_POINTER(offset)) == NULL)
     self->selection = g_list_prepend(self->selection,GUINT_TO_POINTER(offset));
@@ -406,7 +477,7 @@ static void rpm_package_list_select_row(RpmPackageList *self, gint row,
 static void rpm_package_list_unselect_row(RpmPackageList *self, gint row,
 					  gint column, GdkEventButton *event) {
   guint offset;
-  if (event == NULL) return;
+  if (self->mode != RPM_PACKAGE_CLIST) return;
   offset =GPOINTER_TO_UINT(gtk_clist_get_row_data(GTK_CLIST(self->clist),row));
   self->selection = g_list_remove(self->selection, GUINT_TO_POINTER(offset));
   offset = gnome_icon_list_find_icon_from_data(GNOME_ICON_LIST(self->ilist),
@@ -420,7 +491,11 @@ static void rpm_package_list_select_icon(RpmPackageList *self, gint icon,
 					 GdkEvent *event) {
   guint offset;
 
-  if (event == NULL) return;
+  /* Avoid recursive callbacks from maintaining the invisible window 
+     context */
+     
+  if (self->mode == RPM_PACKAGE_CLIST) return;
+
   offset = GPOINTER_TO_UINT(gnome_icon_list_get_icon_data(
 				GNOME_ICON_LIST(self->ilist), icon));
   if (g_list_find(self->selection, GUINT_TO_POINTER(offset)) == NULL)
@@ -431,18 +506,19 @@ static void rpm_package_list_select_icon(RpmPackageList *self, gint icon,
     gtk_clist_select_row(GTK_CLIST(self->clist), offset, -1);
   gtk_signal_emit(GTK_OBJECT(self), rpmpkg_signals[SELECTION_CHANGED]);
 }
+
 static void rpm_package_list_unselect_icon(RpmPackageList *self, gint icon,
 					   GdkEvent *event) {
   guint offset;
+  if (self->mode == RPM_PACKAGE_CLIST) return;
 
-  if (event == NULL) return;
   offset = GPOINTER_TO_UINT(gnome_icon_list_get_icon_data(
 				GNOME_ICON_LIST(self->ilist), icon));
   self->selection = g_list_remove(self->selection, GUINT_TO_POINTER(offset));
   offset = gtk_clist_find_row_from_data(GTK_CLIST(self->clist),
 					GUINT_TO_POINTER(offset));
   if (offset >= 0)
-    gtk_clist_select_row(GTK_CLIST(self->clist), offset, -1);
+    gtk_clist_unselect_row(GTK_CLIST(self->clist), offset, -1);
   gtk_signal_emit(GTK_OBJECT(self), rpmpkg_signals[SELECTION_CHANGED]);
 }
 
