@@ -38,6 +38,8 @@ static void rpm_props_box_apply(GnomePropertyBox *self, gint page_num);
 static void rpm_props_box_class_init(RpmPropsBoxClass *klass);
 static void rpm_props_box_init(RpmPropsBox *box);
 
+extern GnomeApp *gnorpm_app;
+
 guint rpm_props_box_get_type(void) {
   static guint propsbox_type = 0;
   if (!propsbox_type) {
@@ -63,57 +65,7 @@ static void rpm_props_box_class_init(RpmPropsBoxClass *klass) {
   GNOME_PROPERTY_BOX_CLASS(klass)->apply = rpm_props_box_apply;
 }
 
-#ifdef HAVE_RPM_3_0
-typedef struct {
-  gint32 interfaceFlags, transFlags, probFilter;
-  const gchar *tag, *string, *help_string;
-} RpmFlagInfo;
-const static RpmFlagInfo prop_flags[] = {
-    /* install options */
-  {INTER_NODEPS,  0, 0, "NoDependencies", N_("No dependency checks"),
-   N_("Do not check for dependencies on other rpms. Equivalent to --nodeps")},
-  {INTER_NOORDER, 0, 0, "NoReorder", N_("No reordering"),
-   N_("Do not reorder package installation to satisfy dependencies. Equivalent to --noorder")},
-  {0, RPMTRANS_FLAG_NOSCRIPTS, 0, "NoScripts",
-   N_("Don't run scripts"),
-   N_("Do not run the pre and post install scripts. Equivalent to --noscripts.")}, 
-  /* upgrade options */
-  {0, 0, RPMPROB_FILTER_REPLACEPKG, "ReplacePkgs",
-   N_("Allow replacement of packages"), 
-   N_("Replace packages with a new copy of itself. Equivalent to --replacepkgs.")},
-  {0, 0, RPMPROB_FILTER_REPLACEOLDFILES, "ReplaceFiles",
-   N_("Allow replacement of files"),
-   N_("Replace files owned by another package. Equivalent to --replacefiles.")},
-  {0, 0, RPMPROB_FILTER_OLDPACKAGE, "UpgradeToOld",
-   N_("Allow upgrade to old version"),
-   N_("Upgrade packages to an older package if need be. Equivalent to --oldpackage")},
-  {0, RPMTRANS_FLAG_KEEPOBSOLETE, 0, "KeepObsolete",
-   N_("Keep packages made obsolete"),
-   N_("If a package has been obsoleted, keep it anyway.")},
-  /* file options */
-  {0, RPMTRANS_FLAG_NODOCS, 0, "NoDocumentation",
-   N_("Don't install documentation"),
-   N_("Do not install any of the doc files. Equivalent to --excludedocs.")},
-  {0, RPMTRANS_FLAG_ALLFILES, 0, "AllFiles",
-   N_("Install all files"),
-   N_("Install all files")},
-  /* database options */
-  {0, RPMTRANS_FLAG_JUSTDB, RPMTRANS_FLAG_JUSTDB, "JustDB",
-   N_("Just update database"),
-   N_("Do not change any files, just update the database as though you did. Equivalent to --justdb") },
-  {0, RPMTRANS_FLAG_TEST, RPMTRANS_FLAG_TEST, "JustTest",
-   N_("Just test"),
-   N_("Do not actually change anything, just test.  Equivalent to --test") },
-  /* architecture checks */
-  {0, 0, RPMPROB_FILTER_IGNOREARCH, "NoArchCheck",
-   N_("Don't check package architecture"), 
-   N_("Do not check the system and rpm architectures. Equivalent to --ignorearch")},
-  {0, 0, RPMPROB_FILTER_IGNOREOS, "NoOSCheck",
-   N_("Don't check package OS"),
-   N_("Do not check the package OS field. Equivalent to --ignoreos")},
-};
-
-#else /* !HAVE_RPM_3_0 */
+#ifdef HAVE_RPM_2_5
 
 typedef struct {
   gint32 interfaceFlags, installFlags, uninstallFlags;
@@ -163,7 +115,59 @@ const static RpmFlagInfo prop_flags[] = {
    N_("Don't check package OS"),
    N_("Do not check the package OS field. Equivalent to --ignoreos")},
 };
-#endif /* !HAVE_RPM_3_0 */
+
+#else /* !HAVE_RPM_2_5 */
+
+typedef struct {
+  gint32 interfaceFlags, transFlags, probFilter;
+  const gchar *tag, *string, *help_string;
+} RpmFlagInfo;
+const static RpmFlagInfo prop_flags[] = {
+    /* install options */
+  {INTER_NODEPS,  0, 0, "NoDependencies", N_("No dependency checks"),
+   N_("Do not check for dependencies on other rpms. Equivalent to --nodeps")},
+  {INTER_NOORDER, 0, 0, "NoReorder", N_("No reordering"),
+   N_("Do not reorder package installation to satisfy dependencies. Equivalent to --noorder")},
+  {0, RPMTRANS_FLAG_NOSCRIPTS, 0, "NoScripts",
+   N_("Don't run scripts"),
+   N_("Do not run the pre and post install scripts. Equivalent to --noscripts.")}, 
+  /* upgrade options */
+  {0, 0, RPMPROB_FILTER_REPLACEPKG, "ReplacePkgs",
+   N_("Allow replacement of packages"), 
+   N_("Replace packages with a new copy of itself. Equivalent to --replacepkgs.")},
+  {0, 0, RPMPROB_FILTER_REPLACEOLDFILES, "ReplaceFiles",
+   N_("Allow replacement of files"),
+   N_("Replace files owned by another package. Equivalent to --replacefiles.")},
+  {0, 0, RPMPROB_FILTER_OLDPACKAGE, "UpgradeToOld",
+   N_("Allow upgrade to old version"),
+   N_("Upgrade packages to an older package if need be. Equivalent to --oldpackage")},
+  {0, RPMTRANS_FLAG_KEEPOBSOLETE, 0, "KeepObsolete",
+   N_("Keep packages made obsolete"),
+   N_("If a package has been obsoleted, keep it anyway.")},
+  /* file options */
+  {0, RPMTRANS_FLAG_NODOCS, 0, "NoDocumentation",
+   N_("Don't install documentation"),
+   N_("Do not install any of the doc files. Equivalent to --excludedocs.")},
+  {0, RPMTRANS_FLAG_ALLFILES, 0, "AllFiles",
+   N_("Install all files"),
+   N_("Install all files")},
+  /* database options */
+  {0, RPMTRANS_FLAG_JUSTDB, RPMTRANS_FLAG_JUSTDB, "JustDB",
+   N_("Just update database"),
+   N_("Do not change any files, just update the database as though you did. Equivalent to --justdb") },
+  {0, RPMTRANS_FLAG_TEST, RPMTRANS_FLAG_TEST, "JustTest",
+   N_("Just test"),
+   N_("Do not actually change anything, just test.  Equivalent to --test") },
+  /* architecture checks */
+  {0, 0, RPMPROB_FILTER_IGNOREARCH, "NoArchCheck",
+   N_("Don't check package architecture"), 
+   N_("Do not check the system and rpm architectures. Equivalent to --ignorearch")},
+  {0, 0, RPMPROB_FILTER_IGNOREOS, "NoOSCheck",
+   N_("Don't check package OS"),
+   N_("Do not check the package OS field. Equivalent to --ignoreos")},
+};
+
+#endif /* !HAVE_RPM_2_5 */
 
 const int num_prop_flags = sizeof(prop_flags) / sizeof(RpmFlagInfo);
 
@@ -182,7 +186,7 @@ static void rpm_props_box_init(RpmPropsBox *self) {
   GtkTooltips *tooltip;
   gchar *text;
   gchar *distTitles[4] = { N_("Name"), N_("ID"), N_("Rating"), N_("mirror") };
-  GList *tmp;
+  GList *tmp, *metadataList;
   GdkColor colour;
   gint i;
 
@@ -191,10 +195,10 @@ static void rpm_props_box_init(RpmPropsBox *self) {
 
   tooltip = gtk_tooltips_new();
 
-#ifdef HAVE_RPM_3_0
-  self->interfaceFlags = self->transFlags = self->probFilter = 0;
-#else
+#ifdef HAVE_RPM_2_5
   self->interfaceFlags = self->installFlags = self->uninstallFlags = 0;
+#else
+  self->interfaceFlags = self->transFlags = self->probFilter = 0;
 #endif
   self->flags = g_new(GtkWidget *, num_prop_flags);
   /* create the behviour page */
@@ -396,6 +400,10 @@ static void rpm_props_box_init(RpmPropsBox *self) {
 				 "/mnt/cdrom/RedHat/RPMS:"
 				 "/mnt/cdrom/SRPMS:"
 				 "/mnt/cdrom/RPMS:"
+				 "/mnt/cdrom/i386:"
+				 "/mnt/cdrom/alpha:"
+				 "/mnt/cdrom/sparc:"
+				 "/mnt/cdrom/noarch:"
 				 "/usr/src/redhat/RPMS/i386:"
 				 "/usr/src/redhat/RPMS/alpha:"
 				 "/usr/src/redhat/RPMS/sparc:"
@@ -580,9 +588,16 @@ static void rpm_props_box_init(RpmPropsBox *self) {
   gtk_tooltips_set_tip(tooltip, self->metadataServer,
 		       _("The server used to download metadata from"), NULL);
   gtk_combo_set_value_in_list(GTK_COMBO(self->metadataServer), FALSE, FALSE);
-  gtk_combo_set_popdown_strings(GTK_COMBO(self->metadataServer),
-				metadataGetList());
-  text = gnome_config_get_string("/gnorpm/rpmfind/server=http://rufus.w3.org/linux/RDF");
+  metadataList = metadataGetList();
+  if (metadataList)
+    gtk_combo_set_popdown_strings(GTK_COMBO(self->metadataServer),
+				  metadataList);
+
+  if(access("/etc/redhat-release", F_OK)==0)
+    text = gnome_config_get_string("/gnorpm/rpmfind/server=http://www.redhat.com/RDF");
+  else
+    text = gnome_config_get_string("/gnorpm/rpmfind/server=http://rufus.w3.org/linux/RDF");
+
   gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(self->metadataServer)->entry), text);
   g_free(text);
   gtk_signal_connect_object(GTK_OBJECT(GTK_COMBO(self->metadataServer)->entry),
@@ -742,17 +757,23 @@ static void rpm_props_box_init(RpmPropsBox *self) {
   gtk_clist_set_sort_column(GTK_CLIST(self->distClist), 0);
   gtk_clist_set_sort_type(GTK_CLIST(self->distClist), GTK_SORT_ASCENDING);
   gtk_clist_freeze(GTK_CLIST(self->distClist));
+  gtk_object_ref(GTK_OBJECT(gnorpm_app));
   for (tmp = distribGetList(); tmp; tmp = tmp->next) {
-    rpmDistrib *distrib = distribInfoGet(tmp->data);
-    gchar *row[4], buf[21];
+	if (!GTK_OBJECT_DESTROYED(GTK_OBJECT(gnorpm_app))) {
+	  rpmDistrib *distrib = distribInfoGet(tmp->data);
+	  gchar *row[4], buf[21];
 
-    g_snprintf(buf, 20, "%d", distrib->rating);
-    row[0] = distrib->name;
-    row[1] = distrib->ID;
-    row[2] = buf;
-    row[3] = distrib->myMirror;
-    gtk_clist_append(GTK_CLIST(self->distClist), row);
+	  g_snprintf(buf, 20, "%d", distrib->rating);
+	  row[0] = distrib->name;
+	  row[1] = distrib->ID;
+	  row[2] = buf;
+	  row[3] = distrib->myMirror;
+	  gtk_clist_append(GTK_CLIST(self->distClist), row);
+	}
+	else
+		return;
   }
+  gtk_object_unref(GTK_OBJECT(gnorpm_app));
   gtk_clist_sort(GTK_CLIST(self->distClist));
   gtk_clist_thaw(GTK_CLIST(self->distClist));
   gtk_signal_connect(GTK_OBJECT(self->distClist), "select_row",
@@ -849,17 +870,7 @@ GtkWidget *rpm_props_box_new(void) {
 
 static void rpm_props_box_flag_toggled(GtkToggleButton *cb, RpmPropsBox *self){
   RpmFlagInfo *info = gtk_object_get_user_data(GTK_OBJECT(cb));
-#ifdef HAVE_RPM_3_0
-  if (cb->active) {
-    self->interfaceFlags |= info->interfaceFlags;
-    self->transFlags     |= info->transFlags;
-    self->probFilter     |= info->probFilter;
-  } else {
-    self->interfaceFlags &= ~info->interfaceFlags;
-    self->transFlags     &= ~info->transFlags;
-    self->probFilter     &= ~info->probFilter;
-  }
-#else
+#ifdef HAVE_RPM_2_5
   if (cb->active) {
     self->interfaceFlags |= info->interfaceFlags;
     self->installFlags   |= info->installFlags;
@@ -869,7 +880,17 @@ static void rpm_props_box_flag_toggled(GtkToggleButton *cb, RpmPropsBox *self){
     self->installFlags   &= ~info->installFlags;
     self->uninstallFlags &= ~info->uninstallFlags;
   }
-#endif
+#else	/* !HAVE_RPM_2_5 */
+  if (cb->active) {
+    self->interfaceFlags |= info->interfaceFlags;
+    self->transFlags     |= info->transFlags;
+    self->probFilter     |= info->probFilter;
+  } else {
+    self->interfaceFlags &= ~info->interfaceFlags;
+    self->transFlags     &= ~info->transFlags;
+    self->probFilter     &= ~info->probFilter;
+  }
+#endif	/* !HAVE_RPM_2_5 */
   gnome_property_box_changed(GNOME_PROPERTY_BOX(self));
 }
 
@@ -893,15 +914,15 @@ static GtkWidget *rpm_props_box_fill_frame(RpmPropsBox *self, gchar *name,
     g_snprintf(buf, 1023, "/gnorpm/Flags/%s=false", prop_flags[i].tag);
     state = gnome_config_get_bool(buf);
     if (state) {
-#ifdef HAVE_RPM_3_0
-      self->interfaceFlags |= prop_flags[i].interfaceFlags;
-      self->transFlags     |= prop_flags[i].transFlags;
-      self->probFilter     |= prop_flags[i].probFilter;
-#else
+#ifdef HAVE_RPM_2_5
       self->interfaceFlags |= prop_flags[i].interfaceFlags;
       self->installFlags |= prop_flags[i].installFlags;
       self->uninstallFlags |= prop_flags[i].uninstallFlags;
-#endif
+#else	/* !HAVE_RPM_2_5 */
+      self->interfaceFlags |= prop_flags[i].interfaceFlags;
+      self->transFlags     |= prop_flags[i].transFlags;
+      self->probFilter     |= prop_flags[i].probFilter;
+#endif	/* !HAVE_RPM_2_5 */
     }
     gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(self->flags[i]), state);
     gtk_object_set_user_data(GTK_OBJECT(self->flags[i]),
@@ -975,7 +996,6 @@ static void rpm_props_box_apply(GnomePropertyBox *object, gint page_num) {
   gint i, rows;
   gushort r,g,b,a;
   gchar buf[1024], *text, **list;
-  GList *tmp;
 
   switch (page_num) {
   case RPM_PROPS_FLAGS:
@@ -1009,9 +1029,11 @@ static void rpm_props_box_apply(GnomePropertyBox *object, gint page_num) {
     gnome_config_set_string("/gnorpm/Install/newPkg", text);
     g_free(text);
 
+#ifdef WITH_RPMFIND
     gnome_config_set_string("/gnorpm/rpmfind/rpm-dir",
 		gnome_file_entry_get_full_path(
 				GNOME_FILE_ENTRY(self->downloadDir),TRUE));
+#endif
 
     text = gtk_editable_get_chars(GTK_EDITABLE(self->rpmDirs), 0, -1);
     for (i = 0; text[i] != '\0'; i++)
@@ -1070,7 +1092,6 @@ static void rpm_props_box_apply(GnomePropertyBox *object, gint page_num) {
     rows = GTK_CLIST(self->distClist)->rows;
     for (i = 0; i < rows; i++) {
       gchar *id, *rating, *mirror;
-      rpmDistrib *distrib;
 
       gtk_clist_get_text(GTK_CLIST(self->distClist), i, 1, &id);
       gtk_clist_get_text(GTK_CLIST(self->distClist), i, 2, &rating);
@@ -1090,27 +1111,7 @@ static void rpm_props_box_apply(GnomePropertyBox *object, gint page_num) {
     (* parent_class->apply)(object, page_num);
 }
 
-#ifdef HAVE_RPM_3_0
-void rpm_props_box_get_flags(gint32 *interfaceFlags, gint32 *transFlags,
-			     gint32 *probFilter) {
-  gint i;
-  gchar buf[1024];
-  gint32 inter = 0, trans = 0, filter = 0;
-
-  for (i = 0; i < num_prop_flags; i++) {
-    g_snprintf(buf, 1023, "/gnorpm/Flags/%s=false", prop_flags[i].tag);
-    if (gnome_config_get_bool(buf)) {
-      inter  |= prop_flags[i].interfaceFlags;
-      trans  |= prop_flags[i].transFlags;
-      filter |= prop_flags[i].probFilter;
-    }
-  }
-  if (interfaceFlags) *interfaceFlags = inter;
-  if (transFlags)     *transFlags = trans;
-  if (probFilter)     *probFilter = filter;
-}
-
-#else /* !HAVE_RPM_3_0 */
+#ifdef HAVE_RPM_2_5
 
 void rpm_props_box_get_flags(gint32 *interfaceFlags, gint32 *installFlags,
 			     gint32 *uninstallFlags) {
@@ -1130,4 +1131,26 @@ void rpm_props_box_get_flags(gint32 *interfaceFlags, gint32 *installFlags,
   if (installFlags)   *installFlags = inst;
   if (uninstallFlags) *uninstallFlags = uninst;
 }
-#endif /* !HAVE_RPM_3_0 */
+
+#else /* !HAVE_RPM_2_5 */
+
+void rpm_props_box_get_flags(gint32 *interfaceFlags, gint32 *transFlags,
+			     gint32 *probFilter) {
+  gint i;
+  gchar buf[1024];
+  gint32 inter = 0, trans = 0, filter = 0;
+
+  for (i = 0; i < num_prop_flags; i++) {
+    g_snprintf(buf, 1023, "/gnorpm/Flags/%s=false", prop_flags[i].tag);
+    if (gnome_config_get_bool(buf)) {
+      inter  |= prop_flags[i].interfaceFlags;
+      trans  |= prop_flags[i].transFlags;
+      filter |= prop_flags[i].probFilter;
+    }
+  }
+  if (interfaceFlags) *interfaceFlags = inter;
+  if (transFlags)     *transFlags = trans;
+  if (probFilter)     *probFilter = filter;
+}
+
+#endif /* !HAVE_RPM_2_5 */
