@@ -125,9 +125,9 @@ static void rpm_query_class_init(RpmQueryClass *klass) {
 }
 
 static void rpm_query_init(RpmQuery *self) {
-  static gchar *headers[] = { "D", "C", "S", "Path" };
+  static gchar *headers[] = { N_("D"), N_("C"), N_("S"), N_("Path") };
   GtkTable *table;
-  GtkWidget *w, *box, *swin;
+  GtkWidget *w, *box;
 
   table = GTK_TABLE(self);
 
@@ -340,7 +340,6 @@ GtkWidget *rpm_query_new_from_index(DBHandle *hdl, guint index) {
 
 GtkWidget *rpm_query_new_from_file(DBHandle *hdl, gchar *file) {
   RpmQuery *self;
-  dbiIndexSet matches;
 
   self = gtk_type_new(rpm_query_get_type());
   if (rpm_query_set_from_file(self, hdl, file)) {
@@ -350,11 +349,25 @@ GtkWidget *rpm_query_new_from_file(DBHandle *hdl, gchar *file) {
 
   rpm_query_add_install_button(self);
   db_handle_db_up(hdl);
+
+ {
+#ifdef HAVE_RPM_4_0
+  rpmdbMatchIterator mi;
+  mi = rpmdbInitIterator(hdl->db, RPMDBI_LABEL, self->pkg_name, 0);
+  if (rpmdbGetIteratorCount(mi) > 0)
+    rpm_query_add_upgrade_button(self);
+  rpmdbFreeIterator(mi);
+#else
+  dbiIndexSet matches;
+
   if (!rpmdbFindByLabel(hdl->db, self->pkg_name, &matches)) {
     /* there is a package by this name in the rpm database ... */
     dbiFreeIndexRecord(matches);
     rpm_query_add_upgrade_button(self);
   }
+#endif
+ }
+
   db_handle_db_down(hdl);
   rpm_query_add_checksig_button(self);
   rpm_query_add_close_button(self);
@@ -477,13 +490,28 @@ gint rpm_query_set_from_db(RpmQuery *self, DBHandle *hdl, guint index) {
   g_return_val_if_fail(hdl != NULL, 1);
   db_handle_db_up(hdl);
 
+#ifdef HAVE_RPM_4_0
+  { rpmdbMatchIterator mi;
+    mi = rpmdbInitIterator(hdl->db, RPMDBI_PACKAGES, &index, sizeof(index));
+    h = rpmdbNextIterator(mi);
+    if (!h) {
+      rpmdbFreeIterator(mi);
+      db_handle_db_down(hdl);
+      return 1;
+    }
+    rpm_query_set_from_header(self, h);
+    rpmdbFreeIterator(mi);
+  }
+#else
   h = rpmdbGetRecord(hdl->db, index);
+
   if (!h) {
     db_handle_db_down(hdl);
     return 1;
   }
   rpm_query_set_from_header(self, h);
   headerFree(h);
+#endif
 
   if (self->fname) g_free(self->fname);
   self->fname = NULL;
@@ -629,7 +657,7 @@ static void rpm_query_set_from_header(RpmQuery *self, Header h) {
 
   gtk_clist_freeze(GTK_CLIST(self->files));
   gtk_clist_clear(GTK_CLIST(self->files));
-  if (!headerGetEntry(h, RPMTAG_FILENAMES, NULL, (void **)&path, &count)) {
+  if (!headerGetEntry(h, RPMTAG_OLDFILENAMES, NULL, (void **)&path, &count)) {
     gtk_clist_thaw(GTK_CLIST(self->files));
     return;
   }
